%% --------------- Description ------------------
% This script creates six ANN methods and update Digernes formula based on
% the latest model results from MARINTEK

%% --------------- Clear workspace ------------------
clc
clear all
format long
%clear Cleaned Digernes Estimated Empirical Brake Power Geometry Propulsion Resistance Regression_Digernes Regression_Digernes_results

%% --------------- Defining global constants ------------------
global  g rho v n1 m1 N1 maxS maxP Lwl Model_no Version WL Lpp B T Tap Tfp DispV S iE LCB StL Cm Avs AntP D Cr RTS Fn V J

g       = 9.80665;  % Gravity accelration
rho     = 1025;     % Density salt water
v       = 1.15E-6;  % Kinematic viscosity of water
etta_M  = 0.98;     % Mechanical efficiency in egnine
etta_S  = 0.98;     % Transmission efficiency to engine
H       = 150;      % Estimated hull roughness
C_air   = 0.7;      % Estimated coefficient for fishing vessels
rho_air = 1.2;      % Air density
C_A     = -0.228;   % Empirical correlation coefficient

%% --------------- Read Geometrical values ------------------
HG          = csvread('Hullgeo.csv',1,0);       % File with hull geomety
[n1, n2]    = size(HG);                         % Find number of vessels

for i = 1:n1
    Model_no(i)  = HG(i,1);     % Modelnumber - MARINTEK
    Version(i) = HG(i,2);       % Version nr of model
    WL(i)   = HG(i,3);          % Waterline no.
    Scale(i)= HG(i,4);          % Model scale
    Lpp(i)  = HG(i,6);          % Length between perpendicular      [m]
    Lwl(i)  = HG(i,7);          % Waterline length                  [m]
    B(i)    = HG(i,8);          % Moulded breadth                   [m]
    T(i)    = HG(i,11);         % Depth                             [m]
    Tap(i)  = HG(i,12);         % Depth at AP                       [m]
    Tfp(i)  = HG(i,13);         % Depth at FP                       [m]
    DispV(i)= HG(i,14);         % Displacement volume               [m^3]
    S(i)    = HG(i,15);         % Wetted surface                    [m^2]
    iE(i)   = HG(i,21);         % Half angle of entrance of WL      [deg]
    AT(i)   = HG(i,22);         % Wetted transom?? - Not complete   [m^2]
    LCB(i)  = HG(i,23);         % Longitudinal center of Buoyancy   [m]
    StL(i)  = HG(i,27);         % Styrlast (Designtrim)             [m]
    Cm(i)   = HG(i,29);         % Midship section coefficient
    Avs(i)  = HG(i,32);         % Projected frontal area of ship    [m^2]
    AntP(i) = HG(i,33);         % Number of propellers


    Cp(i)   = DispV(i)/(B(i)*T(i)*Cm(i)*Lpp(i));    % Longitudinal prismatic coefficient
    Cb(i)   = DispV(i)/(B(i)*T(i)*Lpp(i));          % Block coefficient
    iE      = iE + (iE==0)*30;                      % Gives a general entrance angle if none i given [Needed for Oortmerssen]
end

Geometry    = table(Model_no', Version', WL', Lpp', Lwl', B', T', Tap', Tfp', DispV', S', iE', AT', LCB', StL', Cm', Avs', AntP', Cp', Cb', ...
                   'VariableNames', {'Model', 'Ver', 'WL', 'Lpp', 'Lwl', 'B', 'T', 'Tap', 'Tfp', 'DispV', 'S', 'iE', 'AT', 'LCB', 'StL', 'Cm', 'Avs', 'AntP', 'Cp', 'Cb'});

clear       AT Cp Cb n2 HG 

%% --------------- Read Propulsion coefficients ------------------
PR          = csvread('P_res.csv',1,0);         % File with propulsion coefficients
PD          = csvread('Propelldata.csv',1,0);   % File with propell data
[m1, m2]    = size(PR);                         % Find number of propulsion measurements
maxP        = max(PR(:,14));                    % Find maximum measurements for one vessel
a           = 1;

Fn_P        = NaN(n1, maxP);      % Test speed for propulsion
J           = NaN(n1, maxP);      % J-number
V_P         = NaN(n1, maxP);      % Deduced test speed from Fn_P

P_D         = NaN(n1, 1);         % P/D for propeller
D           = NaN(1, n1);         % Propellerdiameter
Z           = NaN(n1, 1);         % Number of propellerblades
AeA0        = NaN(n1, 1);         % Blade area ratio
Duct        = NaN(n1, 1);
n_P         = NaN(n1, maxP);      % Propeller revolutions
t           = NaN(n1, maxP);      % Thrustdeduction
w           = NaN(n1, maxP);      % Wake
nR          = NaN(n1, maxP);      % Etta_R
nD          = NaN(n1, maxP);      % Etta_D
n0          = NaN(n1, maxP);      % Etta_0
for i = 1:m1    
% Because more resistance tests has been performed than propulsion tests, the following is needed:   
    if i > 1        
        if PR(i,1) ~= PR(i-1,1) || PR(i,2) ~= PR(i-1,2) || PR(i,3) ~= PR(i-1,3)
            if Model_no(a) ~= PR(i,1) || Version(a) ~= PR(i,2) || WL(a) ~= PR(i,3)
                a = a +1;
            end
        end
    end

    if PR(i,1) == Model_no(a) && PR(i,2) == Version(a) && PR(i,3) == WL(a)
            [D(a), P_D(a), Z(a), AeA0(a), Duct(a)]  = Propell_data(PR(i,7), PR(i,4), PD);
            n_P(a,1)    = PR(i,17)/60;%/sqrt(PR(i,4));
            t(a,1)      = PR(i,18);
            w(a,1)      = PR(i,19);
            nR(a,1)     = PR(i,20);
            nD(a,1)     = PR(i,21);
            
            n0(a,1)     = PR(i,21) / ( ((1-PR(i,18))/(1-PR(i,19))) *PR(i,20) );            
            Fn_P(a,1)   = (PR(i,16)*0.51444444)/sqrt(Lwl(a)*g);
            J(a,1)      = ( (PR(i,16)*0.51444444)*(1-PR(i,19)) ) / ( (PR(i,17)/60) *D(a)/1000 );
            
            b = 0;
            c = i +1;
            while PR(c,1) == Model_no(a) && PR(c,2) == Version(a) && PR(c,3) == WL(a)
                b = b +1;
                
                n_P(a,b)    = PR(c,17)/60;
                t(a,b)      = PR(c,18);
                w(a,b)      = PR(c,19);
                nR(a,b)     = PR(c,20);
                nD(a,b)     = PR(c,21);
                
                n0(a,b)     = PR(c,21) / ( ((1-PR(c,18))/(1-PR(c,19))) *PR(c,20) ); 
                Fn_P(a,b)   = (PR(c,16)*0.51444444)/sqrt(Lwl(a)*g);
                J(a,b)      = ( (PR(c,16)*0.51444444)*(1-PR(c,19)) ) / ( (PR(c,17)/60) *D(a)/1000 );
                
                c = c +1;
                if c > m1
                    break
                end
            end
            
            a = a +1;
            if a > n1
                break
            end
    end       
end    

for i = 1:n1
    for j = 1:maxP
        V_P(i,j)    = Fn_P(i,j) * sqrt(g*Lwl(i));
    end
end

Propulsion  = table(Model_no', Fn_P, V_P, P_D, D', Z, AeA0, Duct, n_P, t, w, nR, nD, n0, 'VariableNames', {'Model', 'Fn', 'V', 'Pitch', 'Diameter', 'Blade_no', 'AeA0', 'Duct', 'Revolutions', 't', 'wake', 'etta_R', 'etta_D', 'etta_0'});


clear a b c Z AeA0 Duct n_P t w nR nD n0 m2 PR PD P_D Fn_P V_P

%% --------------- Read Resistance coefficients ------------------
SR      = csvread('S_res.csv',1,0);         % File with resistance coefficient
[N1, N2]    = size(SR);                     % Find number of resistance coefficients
maxS        = max(SR(:,6));                 % Find maximum measurements for one vessel
a           = 1;

RTS         = NaN(n1, maxS);                % Total resistance
Cr          = NaN(n1, maxS);
Fn          = NaN(n1, maxS);                % Frounde number
V           = NaN(n1, maxS);                % Full scale speed [m/s]
Cbd         = NaN(n1, maxS);                % Transom stern coefficient

T_B         = NaN(n1, maxS);                % Thrustbelastning

for i = 1:N1
    if SR(i,1) == Model_no(a) && SR(i,2) == Version(a) && SR(i,3) == WL(a)
        Vs          = SR(i,9);
        k0          = SR(i,15);
        Cbd(a,1)    = SR(i,17);
        
        RTS(a,1)    = Modellskalering(SR(i,13), Vs, k0, SR(i,17), Avs(a), S(a), Lwl(a), Geometry.Cb(a), B(a), Tap(a), Tfp(a)); %% Motstandsbenevning 
        Cr (a,1)    = SR(i,13);
        Fn (a,1)    = SR(i,11);
        V  (a,1)    = SR(i,9);
        
        T_B(a,1)    = RTS(a,1)/( 0.5 *rho *V(a,1)^(2) *(pi*0.25*D(a)^(2)) );
        
        %if k0 == 0
        %    V(a,1)  = V(a,1)/0.51444444;               % For  rette opp feil i database
        %end
        
       	b = 1;
       	c = i +1;            
            while SR(c,1) == Model_no(a) && SR(c,2) == Version(a) && SR(c,3) == WL(a)
                b = b +1;
                
                Vs          = SR(c,9);
                k0          = SR(c,15);
                Cbd(a,b)    = SR(c,17);
                
                RTS(a,b) = Modellskalering(SR(c,13), Vs, k0, SR(c,17), Avs(a), S(a), Lwl(a), Geometry.Cb(a), B(a), Tap(a), Tfp(a)); %% Motstandsbenevning 
                Cr (a,b) = SR(c,13);
                Fn (a,b) = SR(c,11);
                V  (a,b) = SR(c,9);
                
                T_B(a,b) = RTS(a,b)/( 0.5 *rho *V(a,b)^(2) *(pi*0.25*D(a)^(2)) );
                
                if k0 == 0
                    V(a,b)  = V(a,b)/0.51444444;               % For  rette opp feil i database
                end
                
                c = c +1;
                if c > N1
                    break
                end
            end
       	a  = a +1;
        if a > n1
         	break
       	end
    end  
end

V   = V*0.51444444;     % Convert velocity to m/s

%Cbd(isnan(Cbd)) = 0;
%Cbd     = mean(Cbd');

Resistance  = table(Model_no', Fn, V, Cr, T_B, Cbd, 'VariableNames', {'Model', 'Fn', 'Ship_Speed', 'C_R', 'Thrust_B', 'C_BD'});

% Estimate ITTC coefficients dependent on speed
for i = 1:n1
    for j = 1:maxS
        V(i,j)      = Fn(i,j) * sqrt(g*Geometry.Lwl(i));
        Rn(i,j)  	= (V(i,j)*Lwl(i))/v;
        C_F(i,j)    = 0.075 /((log10(Rn(i,j)) -2)^2);
        dC_F(i,j)   = (111*(H*V(i,j))^(0.21) - 404) *C_F(i,j)^2;
        C_AA(i,j)   = C_air * (rho_air*Avs(i))/(rho *S(i));
    end
end

ITTC    = table(Rn, C_F, dC_F, C_AA, 'VariableNames', {'Rn', 'Cf', 'dCf', 'Caa'});

clear Vs k0 Cbd Ct a b c N2 SR Rn C_F dF_C C_AA

%% --------------- New Digernes regression ------------------
%Counting number of resistance measurements
Fn0                 = Fn;   
Fn0(isnan(Fn0))     = 0;
Fn0(Fn0~=0)         = 1;
FnSum               = sum(sum(Fn0));
a                   = 0;

for i = 1:n1
    for j = 1:maxS
        if Fn0(i,j) ~= 0
            a = a +1;
            
            Regression_Lwl(a)   = Lwl(i);
            Regression_B(a)     = B(i);
            Regression_T(a)     = T(i);
            Regression_DispV(a) = DispV(i);
            Regression_Fn(a)    = Fn(i,j);
            Regression_RTS(a)   = RTS(i,j);
        end
    end
end

% Makes table with all the input parameters
Regression_Digernes             = table( log(Regression_Lwl./Regression_B)', log(Regression_B./Regression_T)', log(Regression_DispV)', Regression_Fn', log(Regression_RTS)', 'VariableNames', {'Lwl_B', 'B_T', 'Displacement', 'Fn', 'Resistance'});
Digernes_model                  = 'Resistance ~ Lwl_B + B_T + Displacement + Fn';
Regression_Digernes_results     = fitlm(Regression_Digernes, Digernes_model);

% Test precicion of old regression
clear Digernes

[Di]    = Digernes();
Di      = Di.*1000;

Di_RTS          = RTS;
Di(isnan(Di))   = 0;
Di_RTS(isnan(Di_RTS))   = 0;

% Calculate R-squared
SSE     = sum(sum((Di - Di_RTS).^2));
SST     = sum(sum((Di_RTS - mean(mean(Di_RTS))).^2));
 
Digernes.Rsquared  = 1 - (SSE/SST);

% Calculate Root Mean Square
RMS     = sqrt(SSE/n1);

Digernes.RMS    = RMS;
  
clear a Regression_Lwl Regression_B Regression_T Regression_DispV Regression_Fn Regression_RTS Digernes_model Di_RTS SSE SST RMS Di Fn0 FnSum
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          
%% --------------- Interpolate values ------------------

% Mean values seems to give best robustness and accuracy
for i = 1:n1
    t_temp  = Propulsion.t(i,:);
    w_temp  = Propulsion.wake(i,:);
    nH_temp = (1-t_temp)./(1-w_temp);
    nR_temp = Propulsion.etta_R(i,:);
    nD_temp = Propulsion.etta_D(i,:);
    n0_temp = Propulsion.etta_0(i,:);
    
    t_temp (isnan(t_temp ))     = [];
    w_temp (isnan(w_temp ))     = [];
    nH_temp(isnan(nH_temp))     = [];
    nR_temp(isnan(nR_temp))     = [];
    nD_temp(isnan(nD_temp))     = [];
    n0_temp(isnan(n0_temp))     = [];
    
    t_mean (i)  = mean(t_temp );
    w_mean (i)  = mean(w_temp );
    nH_mean(i)  = mean(nH_temp);
    nR_mean(i)  = mean(nR_temp);
    nD_mean(i)  = mean(nD_temp);
    n0_mean(i)  = mean(n0_temp);
end

% Save the interpolated values in a table
Interpolated    = table(t_mean', w_mean', nH_mean', nR_mean', nD_mean', n0_mean', 'VariableNames', {'t', 'w', 'etta_H', 'etta_R', 'etta_D', 'etta_0'});

clear t_temp t_mean w_temp w_mean nH_temp nH_mean nR_temp nR_mean nD_temp nD_mean n0_temp n0_mean

%% --------------- Clean data for ANN ------------------

% Here, the plan is to remove inf, NaN and zero values in the dataset to
% avoid any disturbances in the final methods. Because there are different
% data missing for each output, all the variables has to ble cleaned for
% each ANN method

Cleaned = struct;

% --  t  --
thrust  = table(AntP', Geometry.Cb, LCB', B', T', Lwl', D', Geometry.AT, Interpolated.t, (1:n1)', 'VariableNames', {'AntP', 'Cb', 'LCB', 'B', 'T', 'Lwl', 'D', 'AT', 't', 'Serial'});
thrust  (D' == 0, :)	= [];

Cleaned.thrust          = thrust;

% --  w  --
wake 	= table(AntP', Geometry.Cb, LCB', B', T', Lwl', D', Geometry.AT, Interpolated.w, (1:n1)', 'VariableNames', {'AntP', 'Cb', 'LCB', 'B', 'T', 'Lwl', 'D', 'AT', 'w', 'Serial'});
wake 	(D' == 0, :)            = [];

Cleaned.wake       	= wake;

% --  etta_H  --
nH 	= table(AntP', Geometry.Cb, LCB', B', T', Lwl', D', Geometry.AT, Interpolated.etta_H, (1:n1)', 'VariableNames', {'AntP', 'Cb', 'LCB', 'B', 'T', 'Lwl', 'D', 'AT', 'nH', 'Serial'});
nH 	(D' == 0, :)            = [];

Cleaned.nH       	= nH;

% --  etta_R  --
nR      = table(AntP', Geometry.Cb, D', Propulsion.AeA0, Propulsion.Pitch, Propulsion.Duct, B', T', Lwl', Interpolated.etta_R, (1:n1)', 'VariableNames', {'AntP', 'Cb', 'D', 'AeA0', 'Pitch', 'Duct', 'B', 'T', 'Lwl', 'nR', 'Serial'});
nR      (Propulsion.AeA0 == 0, :)   = [];

Cleaned.nR      = nR;

% --  etta_0  --
n0      = table(J, Propulsion.Pitch, Propulsion.AeA0, Propulsion.Blade_no, D', B', T', Propulsion.etta_0, (1:n1)', 'VariableNames', {'J', 'Pitch', 'AeA0', 'Blade_no', 'D', 'B', 'T', 'n0', 'Serial'});
n0      (Propulsion.AeA0 == 0, :)   = [];

Cleaned.n0      = n0;

% --  etta_D  --
nD      = table(AntP', Geometry.Cb, LCB', B', T', Lwl', D', DispV', Propulsion.AeA0, Propulsion.Pitch, Interpolated.etta_D, (1:n1)', 'VariableNames', {'AntP', 'Cb', 'LCB', 'B', 'T', 'Lwl', 'D', 'DispV', 'AeA0', 'Pitch', 'nD', 'Serial'});
nD      (Propulsion.AeA0 == 0, :)  	= [];
nD      (nD.nD(:,1) == 0, :)      	= [];

Cleaned.nD      = nD;


% Count number of columns in each table
N_thrust    = length(thrust.AntP);
N_wake      = length(wake.AntP);
N_nH        = length(nH.AntP);
N_nR        = length(nR.AntP);
N_n0        = size(n0.J);
N_nD        = length(nD.AntP);

N  	= table(N_thrust, N_wake, N_nH, N_nR, N_n0, N_nD, 'VariableNames', {'thrust', 'wake', 'nH', 'nR', 'n0', 'nD'});

Cleaned.N   = N;

clear thrust wake nH nR n0 nD  N_thrust N_wake N_nH N_nR N_n0 N_nD N

%% --------------- Create ANN for Cr ------------------
%{
% Declear new structure for ANN results
ANN = struct;


ANN_in  = zeros(9, FnSum);   % Dependent on number of input variables
ANN_out = zeros(1, FnSum);   % Dependemt on number of output variables

a   = 0;
% Create input and output vectors
for i = 1:n1
    for j = 1:maxS
        if Fn0(i,j) ~= 0
            a = a +1;
            ANN_in(:,a)     = [Fn(i,j); Lwl(i)/nthroot(DispV(i),3); LCB(i)/Lpp(i); Lwl(i)/Lpp(i); Lwl(i)/B(i); B(i)/T(i); (Tap(i)-Tfp(i))/Lpp(i); Cm(i); Geometry.Cb(i)];          % Parameter set #2
            
            ANN_out(a)      = Cr(i,j);
        end
    end
end

% Create ANN
run ANN_Cr 

a       = 0;
test    = zeros(n1,maxS);

% Save output values from ANN
for i = 1:n1
    for j = 1:maxS
        if Fn0(i,j) ~= 0
            a = a +1;
            test(i,j)     = y(a);
        end
    end
end

TEST        = test ~= 0;
test        = test ./ TEST;

C_R         = table((1:n1)', Fn, test, 'VariableNames', {'Serial', 'Fn', 'Est'});

ANN.Cr      = C_R;

clear C_R Fn0 FnSum a test TEST ANN_in ANN_out y net tr e trainTargets trainPerformance trainFcn testTargets testPerformance valTargets hiddenLayerSize 
%}

%% --------------- Create ANN for thrustdeduction ------------------
%{
ANN_in  = zeros(6, Cleaned.N.thrust);   % Dependent on number of input variables
ANN_out = zeros(1, Cleaned.N.thrust);   % Dependemt on number of output variables


% Create input and output vectors
for i = 1:Cleaned.N.thrust
        ANN_in(:,i)     = [Cleaned.thrust.AntP(i); Cleaned.thrust.Cb(i); Cleaned.thrust.LCB(i); Cleaned.thrust.B(i)/Cleaned.thrust.T(i); Cleaned.thrust.Lwl(i)/Cleaned.thrust.B(i); Cleaned.thrust.D(i)/Cleaned.thrust.Lwl(i)];       % Parameterset #1

        ANN_out(i)      = Cleaned.thrust.t(i);
end

% Create ANN
run ANN_t

test        = y;

TEST        = test ~= 0;
test        = test ./ TEST;

t           = table(Cleaned.thrust.Serial, test', 'VariableNames', {'Serial', 'Est'});

ANN.t       = t;

clear t t0 tSum a test TEST ANN_in ANN_out y net tr e trainTargets trainPerformance trainFcn testTargets testPerformance valTargets hiddenLayerSize 
%}

%% --------------- Create ANN for wake ------------------
%{
ANN_in  = zeros(6, Cleaned.N.wake);   % Dependent on number of input variables
ANN_out = zeros(1, Cleaned.N.wake);   % Dependemt on number of output variables

% Create input and output vectors
for i = 1:Cleaned.N.wake
%        ANN_in(:,i)     = [Cleaned.wake.AntP(i); Cleaned.wake.Cb(i); Cleaned.wake.LCB(i); Cleaned.wake.B(i)/Cleaned.wake.T(i); Cleaned.wake.Lwl(i)/Cleaned.wake.B(i); Cleaned.wake.D(i)/Cleaned.wake.Lwl(i); Cleaned.wake.AT(i)/(Cleaned.wake.B(i)*Cleaned.wake.T(i))];       % Parameterset #1
        ANN_in(:,i)     = [Cleaned.wake.AntP(i); Cleaned.wake.Cb(i); Cleaned.wake.LCB(i); Cleaned.wake.B(i)/Cleaned.wake.T(i); Cleaned.wake.Lwl(i)/Cleaned.wake.B(i); Cleaned.wake.D(i)/Cleaned.wake.Lwl(i)];       % Parameterset #2

        ANN_out(i)      = Cleaned.wake.w(i);
end

% Create ANN
run ANN_w

test        = y;

TEST        = test ~= 0;
test        = test ./ TEST;

wake        = table(Cleaned.wake.Serial, test', 'VariableNames', {'Serial', 'Est'});

ANN.w       = wake;

clear w0 wSum tSum Sum a test wake TEST ANN_in ANN_out y net tr e trainTargets trainPerformance trainFcn testTargets testPerformance valTargets hiddenLayerSize 
%}

%% --------------- Create ANN for etta_H ------------------
%{
ANN_in  = zeros(6, Cleaned.N.nH);   % Dependent on number of input variables
ANN_out = zeros(1, Cleaned.N.nH);   % Dependemt on number of output variables

% Create input and output vectors
for i = 1:Cleaned.N.nH
        ANN_in(:,i)     = [Cleaned.nH.AntP(i); Cleaned.nH.Cb(i); Cleaned.nH.LCB(i); Cleaned.nH.B(i)/Cleaned.nH.T(i); Cleaned.nH.Lwl(i)/Cleaned.nH.B(i); Cleaned.nH.D(i)/Cleaned.nH.Lwl(i)];       % Parameterset #2

        ANN_out(i)      = Cleaned.nH.nH(i);
end

% Create ANN
run ANN_nH

test        = y;

TEST        = test ~= 0;
test        = test ./ TEST;

nH          = table(Cleaned.nH.Serial, test', 'VariableNames', {'Serial', 'Est'});

ANN.nH      = nH;

clear   nH nH0 nHSum a test TEST ANN_in ANN_out y net tr e trainTargets trainPerformance trainFcn testTargets testPerformance valTargets hiddenLayerSize

%}

%% --------------- Create ANN for etta_R ------------------
%{
ANN_in  = zeros(8, Cleaned.N.nR);   % Dependent on number of input variables
ANN_out = zeros(1, Cleaned.N.nR);   % Dependemt on number of output variables

% Create input and output vectors
for i = 1:Cleaned.N.nR
        ANN_in(:,i)     = [Cleaned.nR.AntP(i); Cleaned.nR.Cb(i); Cleaned.nR.D(i); Cleaned.nR.AeA0(i); Cleaned.nR.Duct(i); Cleaned.nR.Pitch(i); Cleaned.nR.B(i)/Cleaned.nR.T(i); Cleaned.nR.Lwl(i)/Cleaned.nR.B(i)];       % Parameterset #1

        ANN_out(i)      = Cleaned.nR.nR(i);
end

% Create ANN
run ANN_nR

test        = y;

TEST        = test ~= 0;
test        = test ./ TEST;

nR        	= table(Cleaned.nR.Serial, test', 'VariableNames', {'Serial', 'Est'});

ANN.nR     	= nR;

clear nR nR0 nRSum a test TEST ANN_in ANN_out y net tr e trainTargets trainPerformance trainFcn testTargets testPerformance valTargets hiddenLayerSize 
%}

%% --------------- Create ANN for etta_0 ------------------
%{
n0                  = Cleaned.n0.n0;   
n0(isnan(n0))       = 0;
n0(n0~=0)           = 1;
nSum                = sum(sum(n0));


ANN_in  = zeros(5, nSum);   % Dependent on number of input variables
ANN_out = zeros(1, nSum);   % Dependemt on number of output variables

a   = 0;
% Create input and output vectors
for i = 1:Cleaned.N.n0(1)
    for j = 1:Cleaned.N.n0(2)
        if n0(i,j) ~= 0
            a = a +1;
            %ANN_in(:,a)     = [Cleaned.n0.J(i,j); Cleaned.n0.Pitch(i); Cleaned.n0.AeA0(i); Cleaned.n0.Blade_no(i); Cleaned.n0.D(i); Cleaned.n0.B(i)/Cleaned.n0.T(i)];       % Parameterset #1
            ANN_in(:,a)     = [Cleaned.n0.J(i,j); Cleaned.n0.Pitch(i); Cleaned.n0.AeA0(i); Cleaned.n0.Blade_no(i); Cleaned.n0.D(i)];       % Parameterset #2

            ANN_out(a)      = Cleaned.n0.n0(i,j);
        end
    end
end


% Create ANN
run ANN_n0

a   = 0;
test    = zeros(Cleaned.N.n0(1),Cleaned.N.n0(2));

for i = 1:Cleaned.N.n0(1)
    for j = 1:Cleaned.N.n0(2)
        if n0(i,j) ~= 0
            a = a +1;
            test(i,j)     = y(a);
        end
    end
end

TEST        = test ~= 0;
test        = test ./ TEST;

n_0        	= table(Cleaned.n0.Serial, test, 'VariableNames', {'Serial', 'Est'});

ANN.n0      = n_0;

clear n_0 n0 nSum a test TEST ANN_in ANN_out y net tr e trainTargets trainPerformance trainFcn testTargets testPerformance valTargets hiddenLayerSize 
%}

%% --------------- Create ANN for etta_D ------------------
%{
ANN_in  = zeros(9, Cleaned.N.nD);   % Dependent on number of input variables
ANN_out = zeros(1, Cleaned.N.nD);   % Dependemt on number of output variables

% Create input and output vectors
for i = 1:Cleaned.N.nD
    %ANN_in(:,i)     = [Cleaned.nD.AntP(i); Cleaned.nD.Cb(i); Cleaned.nD.Lwl(i); Cleaned.nD.B(i);                                    Cleaned.nD.DispV(i);                  Cleaned.nD.AeA0(i)];                         % Parameterset #1 (6)
    %ANN_in(:,i)     = [Cleaned.nD.AntP(i); Cleaned.nD.Cb(i); Cleaned.nD.Lwl(i); Cleaned.nD.B(i); Cleaned.nD.T(i);                   Cleaned.nD.DispV(i);                  Cleaned.nD.AeA0(i); Cleaned.nD.Pitch(i)];    % Parameterset #2 (8)
    %ANN_in(:,i)     = [Cleaned.nD.AntP(i); Cleaned.nD.Cb(i); Cleaned.nD.Lwl(i); Cleaned.nD.B(i); Cleaned.nD.T(i); Cleaned.nD.LCB(i);Cleaned.nD.DispV(i); Cleaned.nD.D(i); Cleaned.nD.AeA0(i); Cleaned.nD.Pitch(i)];    % Parameterset #3 (10)
    
    %ANN_in(:,i)     = [Cleaned.nD.AntP(i); Cleaned.nD.Cb(i); Cleaned.nD.Lwl(i)/Cleaned.nD.B(i);                                  Cleaned.nD.LCB(i); Cleaned.nD.Lwl(i)/nthroot(Cleaned.nD.DispV(i),3);                                                        Cleaned.nD.Pitch(i)];       % Parameterset #4 (6)
    %ANN_in(:,i)     = [Cleaned.nD.AntP(i); Cleaned.nD.Cb(i); Cleaned.nD.Lwl(i)/Cleaned.nD.B(i); Cleaned.nD.B(i)/Cleaned.nD.T(i); Cleaned.nD.LCB(i); Cleaned.nD.Lwl(i)/nthroot(Cleaned.nD.DispV(i),3);                                    Cleaned.nD.AeA0(i); Cleaned.nD.Pitch(i)];       % Parameterset #5 (8)
    ANN_in(:,i)     = [Cleaned.nD.AntP(i); Cleaned.nD.Cb(i); Cleaned.nD.Lwl(i)/Cleaned.nD.B(i); Cleaned.nD.B(i)/Cleaned.nD.T(i); Cleaned.nD.LCB(i); Cleaned.nD.Lwl(i)/nthroot(Cleaned.nD.DispV(i),3); Cleaned.nD.D(i)/Cleaned.nD.Lwl(i); Cleaned.nD.AeA0(i); Cleaned.nD.Pitch(i)];       % Parameterset #6 (9)

 	ANN_out(i)      = Cleaned.nD.nD(i);
end

% Create ANN
run ANN_nD

test        = y;

TEST        = test ~= 0;
test        = test ./ TEST;

nD          = table(Cleaned.nD.Serial, test', 'VariableNames', {'Serial', 'Est'});

ANN.nD      = nD;

clear nD a test TEST ANN_in ANN_out y net tr e trainTargets trainPerformance trainFcn testTargets testPerformance valTargets hiddenLayerSize 

%}
%%{

%% --------------- Estimate form-factor and etta_0 ------------------

% Estimates formfactor with MARINTEKs formula
Fulness     = (Geometry.Cb'./Lwl) .*sqrt(B.*(Geometry.Tap' + Geometry.Tfp'));
k0          = 0.6 .*Fulness + 145 .*Fulness.^(3.5);

% Estimate eta_0 with Wageningen B-series
etta_0      = NaN(n1, maxP);

for i  = 1:n1
    for j = 1:maxP
        [Kq, Kt, etta_0(i,j)]      = Wageningen_coeff(J(i,j), Propulsion.Pitch(i), Propulsion.AeA0(i), Propulsion.Blade_no(i));
    end
end

Estimated   = table(k0', etta_0, 'VariableNames', {'k0', 'n0'});

clear Fulness k0 Kq Kt etta_0

%% --------------- Actually Power needed ------------------

% Calculate the power consumtion to calculate estimation error from
Power   = table(Model_no', RTS, Propulsion.V, Interpolated.etta_D, 'VariableNames', {'Model', 'RTS', 'V', 'nD'});

for i = 1:n1
    for j = 1:maxS
        Pb(i,j)     = (Power.RTS(i,j) *Power.V(i,j)) /(etta_S *Power.nD(i));
    end
end

Power.Pb    = Pb;
Power	(Power.nD == 0, :)   = [];

%% --------------- Empirical wake and thrustdeduction ------------------

for i = 1:n1
    for j = 1:maxP
        ANN_Cr(i,j)      =   myNeuralNetworkFunction_Cr([Fn(i,j); Lwl(i)/nthroot(DispV(i),3); LCB(i)/Lpp(i); Lwl(i)/Lpp(i); Lwl(i)/B(i); B(i)/T(i); (Tap(i)-Tfp(i))/Lpp(i); Cm(i); Geometry.Cb(i)]);
        ANN_t(i,j)       =   myNeuralNetworkFunction_t([AntP(i); Geometry.Cb(i); LCB(i); B(i)/T(i); Lwl(i)/B(i); D(i)/Lwl(i)]);
        ANN_w(i,j)       =   myNeuralNetworkFunction_w([AntP(i); Geometry.Cb(i); LCB(i); B(i)/T(i); Lwl(i)/B(i); D(i)/Lwl(i)]);
        ANN_nH(i,j)      =   myNeuralNetworkFunction_nH([AntP(i); Geometry.Cb(i); LCB(i); B(i)/T(i); Lwl(i)/B(i); D(i)/Lwl(i)]);
        ANN_nR(i,j)      =   myNeuralNetworkFunction_nR([AntP(i); Geometry.Cb(i); D(i); Propulsion.AeA0(i); Propulsion.Duct(i); Propulsion.Pitch(i); B(i)/T(i); Lwl(i)/B(i)]);
        ANN_n0(i,j)      =   myNeuralNetworkFunction_n0([J(i,j); Propulsion.Pitch(i); Propulsion.AeA0(i); Propulsion.Blade_no(i); D(i)]);
        ANN_nD(i,j)      =   myNeuralNetworkFunction_nD([AntP(i); Geometry.Cb(i); Lwl(i)/B(i); B(i)/T(i); LCB(i); Lwl(i)/nthroot(DispV(i),3); D(i)/Lwl(i); Propulsion.AeA0(i); Propulsion.Pitch(i)]);    
    end
end

Empirical  = table(Model_no', Fn, ITTC.Rn, B', Lwl', T', Tap', Tfp', S', DispV', D', AntP', Geometry.Cb, Geometry.Cp, Geometry.Cm, LCB', V, Propulsion.V, 1+Estimated.k0, Estimated.n0, Resistance.C_BD, Cr, ITTC.Cf, ITTC.dCf, ITTC.Caa, Pb, Interpolated.t, Interpolated.w, (1-Interpolated.t)./(1-Interpolated.w), Interpolated.etta_R, Propulsion.etta_0, Interpolated.etta_D, ANN_Cr, ANN_w, ANN_t, ANN_nH, ANN_nR, ANN_n0, ANN_nD, ...
            'VariableNames', {'Model', 'Fn', 'Rn', 'B', 'L', 'T', 'Ta', 'Tf', 'S', 'DispV', 'Dp', 'No_P', 'Cb', 'Cp', 'Cm', 'lcb', 'V', 'V_P' 'k0', 'W_n0', 'C_BD', 'Cr', 'Cf', 'dCf', 'Caa', 'Pb', 't', 'w', 'nH', 'nR', 'n0', 'nD', 'ANN_Cr', 'ANN_w', 'ANN_t', 'ANN_nH', 'ANN_nR', 'ANN_n0', 'ANN_nD'});
Empirical  (D' == 0, :)	= [];

N_Empirical     = length(Empirical.B);
clear w1 w2 w3

for i = 1:N_Empirical
    for j = 1:maxP
        [w1(i,j), w2(i,j), w3(i,j), w4_1(i,j), w4_2(i,j), t1(i,j)] = wake_est(Empirical.B(i), Empirical.L(i), Empirical.T(i), Empirical.Ta(i), Empirical.Tf(i), Empirical.S(i), Empirical.DispV(i), Empirical.Dp(i)/1000, Empirical.No_P(i), Empirical.Cb(i), Empirical.Cp(i), Empirical.Cm(i), Empirical.lcb(i), Empirical.V_P(i,j), Empirical.k0(i), 'V');
    end
end

w3(isnan(w3))   = 0;
w3M             = sum(w3')'./sum(w3'~=0)';

Empirical.w1    = w1;
Empirical.w2    = w2;
Empirical.w3    = w3M;
Empirical.w4_1  = w4_1;
Empirical.w4_2  = w4_2;
Empirical.t1    = t1;

%wake            = Disp('Taylor', 'Harvald', 'Holtrop', 'BSRA_1', 'BSRA_2', 'Holtrop'; 'w1', 'w2', 'w3', 'w4_1', 'w4_2', 't');



clear w1 w2 w3 w3M w4_1 w4_2 t1 ANN_*

%% --------------- Estimate engine power ------------------

P_B     = NaN(n1, maxS);
BrakePower  = table(Pb, V, 'VariableNames', {'Pb', 'V'});
BrakePower2 = table(Empirical.Pb, Empirical.V, 'VariableNames', {'Pb', 'V'});

% #1 Estimate based on Etta_D ANN
for i = 1:n1
    for j = 1:maxS
        C_TS    =   (1 + Estimated.k0(i)) *(ITTC.Cf(i,j) + ITTC.dCf(i,j)) + ...
                    (myNeuralNetworkFunction_Cr([Fn(i,j); Lwl(i)/nthroot(DispV(i),3); LCB(i)/Lpp(i); Lwl(i)/Lpp(i); Lwl(i)/B(i); B(i)/T(i); (Tap(i)-Tfp(i))/Lpp(i); Cm(i); Geometry.Cb(i)]) + ...
                    Resistance.C_BD(i,j) + C_A )*1E-3 + ITTC.Caa(i,j);
        R_TS    =   C_TS * 0.5*rho*(V(i,j)^2)*S(i); 
        Etta_D  =   myNeuralNetworkFunction_nD([AntP(i); Geometry.Cb(i); Lwl(i)/B(i); B(i)/T(i); LCB(i); Lwl(i)/nthroot(DispV(i),3); D(i)/Lwl(i); Propulsion.AeA0(i); Propulsion.Pitch(i)]);
        P_B(i,j)=   R_TS *Propulsion.V(i,j) /(etta_S *Etta_D);
    end
end

BrakePower.No1 = P_B;


clear C_TS R_TS Etta_D P_B
P_B     = NaN(n1, maxS);

% #2 Estimate based on wake and thrust deduction from ANN
for i = 1:n1
    for j = 1:maxS
        C_TS    =   (1 + Estimated.k0(i)) *(ITTC.Cf(i,j) + ITTC.dCf(i,j)) + ...
                    (myNeuralNetworkFunction_Cr([Fn(i,j); Lwl(i)/nthroot(DispV(i),3); LCB(i)/Lpp(i); Lwl(i)/Lpp(i); Lwl(i)/B(i); B(i)/T(i); (Tap(i)-Tfp(i))/Lpp(i); Cm(i); Geometry.Cb(i)]) + ...
                    Resistance.C_BD(i,j) + C_A )*1E-3 + ITTC.Caa(i,j);
        R_TS    =   C_TS * 0.5*rho*(V(i,j)^2)*S(i); 
        t       =   myNeuralNetworkFunction_t([AntP(i); Geometry.Cb(i); LCB(i); B(i)/T(i); Lwl(i)/B(i); D(i)/Lwl(i)]);
        w       =   myNeuralNetworkFunction_w([AntP(i); Geometry.Cb(i); LCB(i); B(i)/T(i); Lwl(i)/B(i); D(i)/Lwl(i)]);
        Etta_R  =   myNeuralNetworkFunction_nR([AntP(i); Geometry.Cb(i); D(i); Propulsion.AeA0(i); Propulsion.Duct(i); Propulsion.Pitch(i); B(i)/T(i); Lwl(i)/B(i)]);
        Etta_0  =   myNeuralNetworkFunction_n0([J(i,j); Propulsion.Pitch(i); Propulsion.AeA0(i); Propulsion.Blade_no(i); D(i)]);
        Etta_D  =   ((1-t)/(1-w)) *Etta_R *Etta_0;
        P_B(i,j)=   R_TS *Propulsion.V(i,j) /(etta_S *Etta_D);
    end
end

BrakePower.No2 = P_B;


clear C_TS R_TS t w Etta_R Etta_0 Etta_D P_B
P_B     = NaN(N_Empirical, maxS);

% #3 Estimate based on the best empirical methods taylor (t), Holtrop (w),
% Wageningen (n0), nD=Mean value. Cr (Digernes)for i = 1:n1
R_TS    =   Digernes2();

for i = 1:N_Empirical
    for j = 1:maxS
        %C_TS    =   (1 + Empirical.k0(i)) *(Empirical.Cf(i,j) + Empirical.dCf(i,j)) + ...
        %            (Empirical.Cr(i,j) + Empirical.C_BD(i,j) + C_A )*1E-3 + Empirical.Caa(i,j);
        %R_TS    =   C_TS * 0.5*rho*(Empirical.V(i,j)^2)*Empirical.S(i); 
        t       =   Empirical.t1(i,j);
        w       =   Empirical.w1(i,j);
        Etta_R  =   mean(Empirical.nR);
        Etta_D  =   ((1-t)/(1-w)) *Etta_R *Empirical.W_n0(i,j);
        P_B(i,j)=   R_TS(i,j) *Empirical.V_P(i,j) /(etta_S *Etta_D);
    end
end

BrakePower2.No3 = P_B;

clear C_TS R_TS t w Etta_R Etta_D P_B

%% --------------- Calculate Error ------------------

% Calculates all the errors and saves the result in the structure "Error"

Error = struct;

% Mean error for one propeller

thrust          = Empirical.t;
thrust(Empirical.No_P == 2) = [];
wake            = Empirical.w;
wake(Empirical.No_P == 2) = [];
rotative        = Empirical.nR;
rotative(Empirical.No_P == 2) = [];

n_Empirical     = length(thrust);

error_Mean_t     = (mean(thrust) - thrust) ./ thrust;
std_Mean_t       = std(error_Mean_t);
SSE             = sum((thrust - mean(thrust)).^2);
SST             = sum((thrust - mean(thrust)).^2);
R2_Mean_t        = 1 - (SSE/SST);
RMS_Mean_t       = sqrt(SSE/n_Empirical);

% figure(19)
% plot(error_Mean_t)
% title('Mean t')

error_Mean_w     = (mean(wake) - wake) ./ wake;
std_Mean_w       = std(error_Mean_w);
SSE             = sum((wake - mean(wake)).^2);
SST             = sum((wake - mean(wake)).^2);
R2_Mean_w        = 1 - (SSE/SST);
RMS_Mean_w       = sqrt(SSE/n_Empirical);

% figure(20)
% plot(error_Mean_w)
% title('Mean w')

error_Mean_nR     = (mean(rotative) - rotative) ./ rotative;
std_Mean_nR       = std(error_Mean_nR);
SSE             = sum((rotative - mean(rotative)).^2);
SST             = sum((rotative - mean(rotative)).^2);
R2_Mean_nR        = 1 - (SSE/SST);
RMS_Mean_nR       = sqrt(SSE/n_Empirical);

% figure(21)
% plot(error_Mean_nR)
% title('Mean nR')

Mean_e        = table([mean(thrust); 100*mean(error_Mean_t); std_Mean_t; R2_Mean_t; RMS_Mean_t], [mean(wake); 100*mean(error_Mean_w); std_Mean_w; R2_Mean_w; RMS_Mean_w], [mean(rotative); 100*mean(error_Mean_nR); std_Mean_nR; R2_Mean_nR; RMS_Mean_nR], ...
                    'VariableNames', {'t', 'w', 'nR'}, 'RowNames', {'Mean', 'M_error [%]', 'std', 'R2', 'RMS'});
Error.Mean   = Mean_e;

clear thrust wake rotative n_Empirical

% ANN error

error_ANN_Cr  	= (Empirical.ANN_Cr - Empirical.Cr) ./ Empirical.Cr;
                  error_ANN_Cr(isnan(error_ANN_Cr))   = 0;
                  Empirical.Cr(isnan(Empirical.Cr))   = 0;
                  Empirical.ANN_Cr(isnan(Empirical.ANN_Cr))     = 0;
std_ANN_Cr    	= std(sum(error_ANN_Cr')'./sum(error_ANN_Cr'~=0)');
SSE           	= sum(sum((Empirical.Cr - Empirical.ANN_Cr).^2));
SST             = sum(sum((Empirical.Cr - mean(sum(Empirical.Cr')'./sum(Empirical.Cr'~=0)')).^2));
R2_ANN_Cr      	= 1 - (SSE/SST);
RMS_ANN_Cr     	= sqrt(SSE/N_Empirical);

% figure(22)
% surf(error_ANN_Cr)
% title('ANN Cr')

error_ANN_t     = (Empirical.ANN_t(:,1) - Empirical.t) ./ Empirical.t;
std_ANN_t       = std(error_ANN_t);
SSE             = sum((Empirical.t - Empirical.ANN_t(:,1)).^2);
SST             = sum((Empirical.t - mean(Empirical.t)).^2);
R2_ANN_t        = 1 - (SSE/SST);
RMS_ANN_t       = sqrt(SSE/N_Empirical);

% figure(23)
% plot(error_ANN_t)
% title('ANN t')

error_ANN_w     = (Empirical.ANN_w(:,1) - Empirical.w) ./ Empirical.w;
std_ANN_w       = std(error_ANN_w);
SSE             = sum((Empirical.w - Empirical.ANN_w(:,1)).^2);
SST             = sum((Empirical.w - mean(Empirical.w)).^2);
R2_ANN_w        = 1 - (SSE/SST);
RMS_ANN_w       = sqrt(SSE/N_Empirical);

% figure(24)
% plot(error_ANN_t)
% title('ANN w')

error_ANN_nR   	= (Empirical.ANN_nR(:,1) - Empirical.nR) ./ Empirical.nR;
std_ANN_nR     	= std(error_ANN_nR);
SSE             = sum((Empirical.nR - Empirical.ANN_nR(:,1)).^2);
SST             = sum((Empirical.nR - mean(Empirical.nR)).^2);
R2_ANN_nR       = 1 - (SSE/SST);
RMS_ANN_nR      = sqrt(SSE/N_Empirical);

% figure(25)
% plot(error_ANN_nR)
% title('ANN nR')

error_ANN_nH   	= (Empirical.ANN_nH(:,1) - Empirical.nH) ./ Empirical.nH;
std_ANN_nH     	= std(error_ANN_nH);
SSE             = sum((Empirical.nH - Empirical.ANN_nH(:,1)).^2);
SST             = sum((Empirical.nH - mean(Empirical.nH)).^2);
R2_ANN_nH       = 1 - (SSE/SST);
RMS_ANN_nH      = sqrt(SSE/N_Empirical);

% figure(26)
% plot(error_ANN_nH)
% title('ANN nH')

error_ANN_n0    = (Empirical.ANN_n0 - Empirical.n0) ./ Empirical.n0;
                  error_ANN_n0(isnan(error_ANN_n0))   = 0;
                  error_ANN_n0(isinf(error_ANN_n0))   = 0;
                  Empirical.n0(isnan(Empirical.n0))   = 0;
                  Empirical.ANN_n0(isnan(Empirical.ANN_n0))     = 0;
temp            = sum(error_ANN_n0')'./sum(error_ANN_n0'~=0)';
                  temp(isnan(temp))                   = 0;
std_ANN_n0      = sqrt((1 ./(sum(sum(error_ANN_n0~=0)) -1)) .* sum( (temp - (sum(sum(error_ANN_n0))./sum(sum(error_ANN_n0~=0)))).^2 ) );
SSE             = sum(sum((Empirical.n0 - Empirical.ANN_n0).^2));
SST             = sum(sum((Empirical.n0 - sum(sum(error_ANN_n0))./sum(sum(error_ANN_n0~=0))).^2));
R2_ANN_n0       = 1 - (SSE/SST);
RMS_ANN_n0      = sqrt(SSE/N_Empirical);

 figure(27)
surf(error_ANN_n0)
title('ANN n0', 'LineStyle', 'none')

% To fix some missing results in n0
temp2           = sum(error_ANN_n0')';
temp2(temp2 == 0)   = [];
temp3           = sum(error_ANN_n0'~=0)';
temp3(temp3 == 0)   = [];

error_ANN_nD    = (Empirical.ANN_nD(:,1) - Empirical.nD) ./ Empirical.nD;
                  error_ANN_nD(isinf(error_ANN_nD))     = 0;
std_ANN_nD      = sqrt((1 ./(sum(error_ANN_nD~=0) -1)) .* sum( (error_ANN_nD - (sum(error_ANN_nD)./sum(error_ANN_nD~=0))).^2 ) );
SSE             = sum((Empirical.nD - Empirical.ANN_nD(:,1)).^2);
SST             = sum((Empirical.nD - sum(error_ANN_nD)./sum(error_ANN_nD~=0)).^2);
R2_ANN_nD       = 1 - (SSE/SST);
RMS_ANN_nD      = sqrt(SSE/N_Empirical);

% figure(28)
% plot(error_ANN_nD)
% title('Mean nD')

ANNe        = table([100*mean(sum(error_ANN_Cr')'./sum(error_ANN_Cr'~=0)'); std_ANN_Cr; R2_ANN_Cr; RMS_ANN_Cr], [100*mean(error_ANN_t); std_ANN_t; R2_ANN_t; RMS_ANN_t], [100*mean(error_ANN_w); std_ANN_w; R2_ANN_w; RMS_ANN_w], [100*mean(error_ANN_nR); std_ANN_nR; R2_ANN_nR; RMS_ANN_nR], [100*mean(error_ANN_nH); std_ANN_nH; R2_ANN_nH; RMS_ANN_nH], [100*mean(temp2./temp3); std_ANN_n0; R2_ANN_n0; RMS_ANN_n0], [100*mean(error_ANN_nD); std_ANN_nD; R2_ANN_nD; RMS_ANN_nD], ...
                    'VariableNames', {'Cr', 't', 'w', 'nR', 'nH', 'n0', 'nD'}, 'RowNames', {'M_error [%]', 'std', 'R2', 'RMS'});
Error.ANN   = ANNe;

% Empirical error

error_w1  	= (mean(Empirical.w1')'    - Empirical.w) ./ Empirical.w;
std_w1    	= std(error_w1);
SSE         = sum((mean(Empirical.w1')'  - Empirical.w).^2);
SST         = sum((Empirical.w - mean(mean(Empirical.w'))).^2);
R2_w1      	= 1 - (SSE/SST);
RMS_w1     	= sqrt(SSE/N_Empirical);

error_w2  	= (mean(Empirical.w2')'    - Empirical.w) ./ Empirical.w;
std_w2     	= std(error_w2);
SSE         = sum((mean(Empirical.w2')'  - Empirical.w).^2);
SST         = sum((Empirical.w - mean(mean(Empirical.w'))).^2);
R2_w2      	= 1 - (SSE/SST);
RMS_w2     	= sqrt(SSE/N_Empirical);

error_w3  	= (Empirical.w3    - Empirical.w) ./ Empirical.w;
std_w3     	= std(error_w3);
SSE         = sum((Empirical.w3  - Empirical.w).^2);
SST         = sum((Empirical.w - mean(Empirical.w)).^2);
R2_w3      	= 1 - (SSE/SST);
RMS_w3    	= sqrt(SSE/N_Empirical);

error_w4a  	= (mean(Empirical.w4_1')'  - Empirical.w) ./ Empirical.w;
std_w4a    	= std(error_w4a);
SSE         = sum((mean(Empirical.w4_1')'  - Empirical.w).^2);
SST         = sum((Empirical.w - mean(mean(Empirical.w))).^2);
R2_w4a     	= 1 - (SSE/SST);
RMS_w4a    	= sqrt(SSE/N_Empirical);

error_w4b  	= (mean(Empirical.w4_2')'  - Empirical.w) ./ Empirical.w;
std_w4b    	= std(error_w4b);
SSE         = sum((mean(Empirical.w4_2')'  - Empirical.w).^2);
SST         = sum((Empirical.w - mean(mean(Empirical.w))).^2);
R2_w4b     	= 1 - (SSE/SST);
RMS_w4b   	= sqrt(SSE/N_Empirical);

error_t     = (mean(Empirical.t1')'    - Empirical.t) ./ Empirical.t;
std_t       = std(error_t);
SSE         = sum((mean(Empirical.t1')'  - Empirical.t).^2);
SST         = sum((Empirical.t - mean(mean(Empirical.t))).^2);
R2_t        = 1 - (SSE/SST);
RMS_t       = sqrt(SSE/N_Empirical);

error_W_n0 	= (Empirical.W_n0 - Empirical.n0) ./ Empirical.n0;
                  error_W_n0(isnan(error_W_n0))   = 0;
                  error_W_n0(isinf(error_W_n0))   = 0;
                  Empirical.n0(isnan(Empirical.n0))   = 0;
                  Empirical.W_n0(isnan(Empirical.W_n0))     = 0;
temp     	= sum(error_W_n0')'./sum(error_W_n0'~=0)';
                  temp(isnan(temp))                   = 0;
std_W_n0  	= sqrt((1 ./(sum(sum(error_W_n0~=0)) -1)) .* sum( (temp - (sum(sum(error_W_n0))./sum(sum(error_W_n0~=0)))).^2 ) );
SSE       	= sum(sum((Empirical.n0 - Empirical.W_n0).^2));
SST        	= sum(sum((Empirical.n0 - sum(sum(error_W_n0))./sum(sum(error_W_n0~=0))).^2));
R2_W_n0   	= 1 - (SSE/SST);
RMS_W_n0  	= sqrt(SSE/N_Empirical);

figure(29)
surf(error_W_n0)
title('Wageningen n0')

% To fix some missing results in n0
temp2           = sum(error_W_n0')';
temp2(temp2 == 0)   = [];
temp3           = sum(error_W_n0'~=0)';
temp3(temp3 == 0)   = [];

Empiricale        = table([100*mean(error_w1); std_w1; R2_w1; RMS_w1], [100*mean(error_w2); std_w2; R2_w2; RMS_w2], [100*mean(error_w3); std_w3; R2_w3; RMS_w3], [100*mean(error_w4a); std_w4a; R2_w4a; RMS_w4a], [100*mean(error_w4b); std_w4b; R2_w4b; RMS_w4b], [100*mean(error_t); std_t; R2_t; RMS_t], [100*mean(temp2./temp3); std_W_n0; R2_W_n0; RMS_W_n0], ...
                    'VariableNames', {'w1', 'w2', 'w3', 'w4_1', 'w4_2', 't', 'W_n0'}, 'RowNames', {'M_error [%]', 'std', 'R2', 'RMS'});
Error.Empirical   = Empiricale;

% Power error

BrakePower  (Interpolated.etta_D == 0 , :)  = [];
N_power     = size(BrakePower.No1,1);

% nD from ANN
error_Pd1       = (BrakePower.No1 - BrakePower.Pb) ./ BrakePower.Pb;
                  error_Pd1(isnan(error_Pd1))   = 0;
                  error_Pd1(isinf(error_Pd1))   = 0;
                  BrakePower.Pb(isnan(BrakePower.Pb))   = 0;
                  BrakePower.No1(isnan(BrakePower.No1))     = 0;
temp            = sum(error_Pd1')'./sum(error_Pd1'~=0)';
                  temp(isnan(temp))                   = 0;
std_Pd1         = sqrt((1 ./(sum(sum(error_Pd1~=0)) -1)) .* sum( (temp - (sum(sum(error_Pd1))./sum(sum(error_Pd1~=0)))).^2 ) );
SSE             = sum(sum((BrakePower.Pb - BrakePower.No1).^2));
SST             = sum(sum((BrakePower.Pb - sum(sum(error_Pd1))./sum(sum(error_Pd1~=0))).^2));
R2_Pd1          = 1 - (SSE/SST);
RMS_Pd1         = sqrt(SSE/N_power);

% figure(30)
% surf(error_Pd1)
% title('P_D nD ANN')

% n0 from ANN
error_Pd2       = (BrakePower.No2 - BrakePower.Pb) ./ BrakePower.Pb;
                  error_Pd2(isnan(error_Pd2))   = 0;
                  error_Pd2(isinf(error_Pd2))   = 0;
                  BrakePower.Pb(isnan(BrakePower.Pb))   = 0;
                  BrakePower.No2(isnan(BrakePower.No2))     = 0;
temp            = sum(error_Pd2')'./sum(error_Pd2'~=0)';
                  temp(isnan(temp))                   = 0;
std_Pd2         = sqrt((1 ./(sum(sum(error_Pd2~=0)) -1)) .* sum( (temp - (sum(sum(error_Pd2))./sum(sum(error_Pd2~=0)))).^2 ) );
SSE             = sum(sum((BrakePower.Pb - BrakePower.No2).^2));
SST             = sum(sum((BrakePower.Pb - sum(sum(error_Pd2))./sum(sum(error_Pd2~=0))).^2));
R2_Pd2          = 1 - (SSE/SST);
RMS_Pd2         = sqrt(SSE/N_power);

% figure(31)
% surf(error_Pd2)
% title('P_D n0 ANN')

BrakePower2(Empirical.nD == 0 , :)  = [];
N_power2     = size(BrakePower2.No3,1);

% n0 from Wageningen
error_Pd3       = (BrakePower2.No3 - BrakePower2.Pb) ./ BrakePower2.Pb;
                  error_Pd3(isnan(error_Pd3))   = 0;
                  error_Pd3(isinf(error_Pd3))   = 0;
                  BrakePower2.Pb(isnan(BrakePower2.Pb))   = 0;
                  BrakePower2.No3(isnan(BrakePower2.No3))     = 0;
temp            = sum(error_Pd3')'./sum(error_Pd3'~=0)';
                  temp(isnan(temp))                   = 0;
std_Pd3         = sqrt((1 ./(sum(sum(error_Pd3~=0)) -1)) .* sum( (temp - (sum(sum(error_Pd3))./sum(sum(error_Pd3~=0)))).^2 ) );
SSE             = sum(sum((BrakePower2.Pb - BrakePower2.No3).^2));
SST             = sum(sum((BrakePower2.Pb - sum(sum(error_Pd3))./sum(sum(error_Pd3~=0))).^2));
R2_Pd3          = 1 - (SSE/SST);
RMS_Pd3         = sqrt(SSE/N_power2);

% figure(32)
% surf(error_Pd3)
% title('P_D n0 Wag')

Powere      = table([100*mean(sum(error_Pd1')'./sum(error_Pd1'~=0)'); std_Pd1; R2_Pd1; RMS_Pd1], [100*mean(sum(error_Pd2')'./sum(error_Pd2'~=0)'); std_Pd2; R2_Pd2; RMS_Pd2], [100*mean(sum(error_Pd3')'./sum(error_Pd3'~=0)'); std_Pd3; R2_Pd3; RMS_Pd3], ...
                    'VariableNames', {'nD_ANN', 'n0_ANN', 'n0_Wag'}, 'RowNames', {'M_error [%]', 'std', 'R2', 'RMS'});
Error.Power = Powere;

% figure, plot(BrakePower.V'/0.51444444, error_Pd1')
% title('n_D ANN')
% figure, plot(BrakePower.V'/0.51444444, error_Pd2')
% title('n_0 from ANN')
% figure, plot(BrakePower2.V'/0.51444444, error_Pd3')
% title('n_0 from Wagening')

clear SSE SST Mean_e ANNe Empiricale temp* N_power N_power2
clear error_* std_* R2_* RMS_*
