%% --------------- Description ------------------
% This script is made to present the accuracy of empirical resistance
% prediction methods, and to graphically show if there is any trend in the
% error

% The script also calculate new regression coefficients for 'Digernes
% resistance prediction function', based on it log-linear structure. An
% Artificial Neural Network method is also calculated (Require MATLAB's
% Neural Network toolbox), and exported as (... sett inn formatet ANN
% metoden eksporteres som).

% The script is written to analyze the data for my Master Thesis, but by
% changing the .csv - files 'Hullgeo.csv', 'P_res.csv' and 'S_res.csv' with
% files with the same column structure. Other datasets can be analyzed. 

% The methods used in this script is choosen because of the scope of the
% master thesis, which is to investigate methods for fishing vessels.
%   - Artificial Neural Network from Marintek   (2004)
%   - Digernes resistance prediction function   (1983)
%   - Hollenbach                                (1998)
%   - Holtrop                                   (1984)
%   - Oortmerssen resistane prediction method   (1971)

%% --------------- Clear workspace ------------------
clc
clear all
format long

%% --------------- Open files ------------------
Error_file          = fopen('Mastergrad/Error.txt', 'w');
Error_design_file   = fopen('Mastergrad/Error_design.txt','w');
Error_Fn_file       = fopen('Mastergrad/Error_Fn.txt','w');               
Std_dev_Fn_file     = fopen('Mastergrad/Std_dev_Fn.txt','w');              
Vessel_overview_file= fopen('Mastergrad/Vessel_overview.txt','w');          

%% --------------- Defining global constants ------------------
global  g rho v n1 m1 N1 maxS Lwl Model_no Version WL Lpp B T Tap Tfp DispV S iE LCB Cm Avs AntP P_D D Cr RTS Fn V ANN_in ANN_out

g   = 9.80665;  % Gravity accelration
rho = 1025;     % Density salt water
v   = 1.15E6;   % Kinematic viscosity of water

%% --------------- Read Geometrical values ------------------
HG          = csvread('Hullgeo.csv',1,0);       % File with hull geomety
[n1, n2]    = size(HG);                         % Find number of vessels

for i = 1:n1
    Model_no(i)  = HG(i,1);     % Modelnumber - MARINTEK
    Version(i) = HG(i,2);       % Version nr of model
    WL(i)   = HG(i,3);          % Waterline no.
    Scale(i)= HG(i,4);          % Model scale
    Lpp(i)  = HG(i,6);          % Length between perpendicular      [m]
    Lwl(i)  = HG(i,7);          % Waterline length                  [m]
    B(i)    = HG(i,8);          % Moulded breadth                   [m]
    T(i)    = HG(i,11);         % Depth                             [m]
    Tap(i)  = HG(i,12);         % Depth at AP                       [m]
    Tfp(i)  = HG(i,13);         % Depth at FP                       [m]
    DispV(i)= HG(i,14);         % Displacement volume               [m^3]
    S(i)    = HG(i,15);         % Wetted surface                    [m^2]
    iE(i)   = HG(i,21);         % Half angle of entrance of WL      [deg]
    AT(i)   = HG(i,22);         % Wetted transom?? - Not complete   [m^2]
    LCB(i)  = HG(i,23);         % Longitudinal center of Buoyancy   [m]
    Cm(i)   = HG(i,29);         % Midship section coefficient
    Avs(i)  = HG(i,32);         % Projected frontal area of ship    [m^2]
    AntP(i) = HG(i,33);         % Number of propellers


    Cp(i)   = DispV(i)/(B(i)*T(i)*Cm(i)*Lpp(i));    % Longitudinal prismatic coefficient
    Cb(i)   = DispV(i)/(B(i)*T(i)*Lpp(i));          % Block coefficient
    iE      = iE + (iE==0)*30;                      % Gives a general entrance angle if none i given [Needed for Oortmerssen]
end

Geometry    = table(Model_no', Version', WL', Lpp', Lwl', B', T', Tap', Tfp', DispV', S', iE', AT', LCB', Cm', Avs', AntP', Cp', Cb', ...
                    'VariableNames', {'Model', 'Ver', 'WL', 'Lpp', 'Lwl', 'B', 'T', 'Tap', 'Tfp', 'DispV', 'S', 'iE', 'AT', 'LCB', 'Cm', 'Avs', 'AntP', 'Cp', 'Cb'});
                
clear       AT Cp Cb n2 HG 

%% --------------- Read Propulsion coefficients ------------------
PR          = csvread('P_res.csv',1,0);         % File with propulsion coefficients
PD          = csvread('Propelldata.csv',1,0);   % File with propell data
[m1, m2]    = size(PR);                         % Find number of propulsion measurements
maxP        = max(PR(:,14));                    % Find maximum measurements for one vessel
a           = 1;

Fn_P        = NaN(n1, maxP);      % Test speed for propulsion

P_D         = NaN(n1, 1);         % P/D for propeller
D           = NaN(n1, 1);         % Propellerdiameter
Z           = NaN(n1, 1);         % Number of propellerblades
AeA0        = NaN(n1, 1);         % Blade area ratio
n_P         = NaN(n1, maxP);      % Propeller revolutions
t           = NaN(n1, maxP);      % Thrustdeduction
w           = NaN(n1, maxP);      % Wake
nR          = NaN(n1, maxP);      % Etta_R
nD          = NaN(n1, maxP);      % Etta_D

for i = 1:m1    
% Because more resistance tests has been performed than propulsion tests, the following is needed:   
    if i > 1        
        if PR(i,1) ~= PR(i-1,1) || PR(i,2) ~= PR(i-1,2) || PR(i,3) ~= PR(i-1,3)
            if Model_no(a) ~= PR(i,1) || Version(a) ~= PR(i,2) || WL(a) ~= PR(i,3)
                a = a +1;
            end
        end
    end

    if PR(i,1) == Model_no(a) && PR(i,2) == Version(a) && PR(i,3) == WL(a)
            [D(a), P_D(a), Z(a), AeA0(a)]  = Propell_data(PR(i,7), PR(i,4), PD);
            n_P(a,1)    = PR(i,17);
            t(a,1)      = PR(i,18);
            w(a,1)      = PR(i,19);
            nR(a,1)     = PR(i,20);
            nD(a,1)     = PR(i,21);
            
            Fn_P(a,1)   = (PR(i,16)*0.51444444)/sqrt(Lwl(a)*g);
            
            b = 0;
            c = i +1;
            while PR(c,1) == Model_no(a) && PR(c,2) == Version(a) && PR(c,3) == WL(a)
                b = b +1;
                
                n_P(a,b)    = PR(c,17);
                t(a,b)      = PR(c,18);
                w(a,b)      = PR(c,19);
                nR(a,b)     = PR(c,20);
                nD(a,b)     = PR(c,21);
                
                Fn_P(a,b)   = (PR(c,16)*0.51444444)/sqrt(Lwl(a)*g);
                
                c = c +1;
                if c > m1
                    break
                end
            end
            
            a = a +1;
            if a > n1
                break
            end
            
    end       
end        

Propulsion  = table(Model_no', Fn_P, P_D, D, Z, AeA0, n_P, t, w, nR, nD, 'VariableNames', {'Model', 'Fn', 'Pitch', 'Diameter', 'Blade_no', 'AeA0', 'Revolutions', 't', 'wake', 'etta_R', 'etta_D'});

clear a b c Z AeA0 n_P t w nR nD m2 maxP PR PD Fn_P

%% --------------- Read Resistance coefficients ------------------
SR      = csvread('S_res.csv',1,0);         % File with resistance coefficient
[N1, N2]    = size(SR);                     % Find number of resistance coefficients
maxS        = max(SR(:,6));                 % Find maximum measurements for one vessel
a           = 1;

RTS         = NaN(n1, maxS);                % Total resistance
Cr          = NaN(n1, maxS);
Fn          = NaN(n1, maxS);                % Frounde number
V           = NaN(n1, maxS);                % Full scale speed [m/s]

T_B         = NaN(n1, maxS);                % Thrustbelastning

for i = 1:N1
    if SR(i,1) == Model_no(a) && SR(i,2) == Version(a) && SR(i,3) == WL(a)
        Vs          = SR(i,9);
        k0          = SR(i,15);
        Cbd         = SR(i,17);
        
        RTS(a,1)    = Modellskalering(SR(i,13), Vs, k0, Cbd, Avs(a), S(a), Lwl(a), Geometry.Cb(a), B(a), Tap(a), Tfp(a)); %% Motstandsbenevning 
        Cr (a,1)    = SR(i,13);
        Fn (a,1)    = SR(i,11);
        V  (a,1)    = SR(i,9);
        
        T_B(a,1)    = RTS(a,1)/( 0.5 *rho *V(a,1)^(2) *(pi*0.25*D(a)^(2)) );
        
        %if k0 == 0
        %    V(a,1)  = V(a,1)/0.51444444;               % For  rette opp feil i database
        %end
        
       	b = 1;
       	c = i +1;            
            while SR(c,1) == Model_no(a) && SR(c,2) == Version(a) && SR(c,3) == WL(a)
                b = b +1;
                
                Vs          = SR(c,9);
                k0          = SR(c,15);
                Cbd         = SR(c,17);
                
                RTS(a,b) = Modellskalering(SR(c,13), Vs, k0, Cbd, Avs(a), S(a), Lwl(a), Geometry.Cb(a), B(a), Tap(a), Tfp(a)); %% Motstandsbenevning 
                Cr (a,b) = SR(c,13);
                Fn (a,b) = SR(c,11);
                V  (a,b) = SR(c,9);
                
                T_B(a,b) = RTS(a,b)/( 0.5 *rho *V(a,b)^(2) *(pi*0.25*D(a)^(2)) );
                
                if k0 == 0
                    V(a,b)  = V(a,b)/0.51444444;               % For  rette opp feil i database
                end
                
                c = c +1;
                if c > N1
                    break
                end
            end
       	a  = a +1;
        if a > numel(Model_no)
         	break
       	end
    end  
end

V   = V*0.51444444;     % Convert velocity to m/s

Resistance  = table(Model_no', Fn, V, Cr, T_B, 'VariableNames', {'Model', 'Fn', 'Ship_Speed', 'C_R', 'Thrust_B'});

clear Vs k0 Cbd Ct a b c N2 SR

%% --------------- New Digernes regression ------------------
%Counting number of resistance measurements
Fn0                 = Fn;   
Fn0(isnan(Fn0))     = 0;
Fn0(Fn0~=0)         = 1;
FnSum               = sum(sum(Fn0));
a                   = 0;

for i = 1:n1
    for j = 1:maxS
        if Fn0(i,j) ~= 0
            a = a +1;
            
            Regression_Lwl(a)   = Lwl(i);
            Regression_B(a)     = B(i);
            Regression_T(a)     = T(i);
            Regression_DispV(a) = DispV(i);
            Regression_Fn(a)    = Fn(i,j);
            Regression_RTS(a)   = RTS(i,j);
        end
    end
end

Regression_Digernes             = table( log(Regression_Lwl./Regression_B)', log(Regression_B./Regression_T)', log(Regression_DispV)', Regression_Fn', log(Regression_RTS)', 'VariableNames', {'Lwl_B', 'B_T', 'Displacement', 'Fn', 'Resistance'});
Digernes_model                  = 'Resistance ~ Lwl_B + B_T + Displacement + Fn';
Regression_Digernes_results     = fitlm(Regression_Digernes, Digernes_model);

% Test precicion of old regression
[Di]    = Digernes();
Di      = Di.*1000;

Di_RTS          = RTS;
Di(isnan(Di))   = 0;
Di_RTS(isnan(Di_RTS))   = 0;

% Calculate R-squared
SSE     = sum(sum((Di - Di_RTS).^2));
SST     = sum(sum((Di_RTS - mean(mean(Di_RTS))).^2));
 
Digernes.Rsquared  = 1 - (SSE/SST);

% Calculate Root Mean Square
RMS     = sqrt(SSE/n1);

Digernes.RMS    = RMS;
  
clear a Regression_Lwl Regression_B Regression_T Regression_DispV Regression_Fn Regression_RTS Digernes_model Di_RTS SSE SST RMS Di 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          
%% --------------- Calculation of Digernes & Oortmerssen [Ex] ------------------
%{
[O, Op, Ot, Ow, OnR]  	= Oortmerssen_71();
[Di]                    = Digernes();

Di = Di*1000;

Oortmerssen     = table(O, Op, Ot, Ow, OnR, 'VariableNames', {'Resistance', 'Eff_Power', 't', 'wake', 'etta_R'});
Digernes        = table(Di, 'VariableNames', {'Resistance'});

clear O Op Ot Ow OnR Di
%} 

%% --------------- Read ANN, Holtrop, Hollenbach resistance [Ex] ------------------
%{
ANN_results         = csvread('ANN.csv',0,0);
Holtrop_results     = csvread('Holtrop.csv',0,0);
Hollenbach_results  = csvread('Hollenbach.csv',0,0);

% Converting zeros to NaN
ANN                 = ANN_results ~= 0;
ANN_results         = ANN_results ./ ANN;

Holtrop             = Holtrop_results ~= 0;
Holtrop_results     = Holtrop_results ./ Holtrop;

Hollenbach          = Hollenbach_results ~= 0;
Hollenbach_results  = Hollenbach_results ./ Hollenbach;

ANN         = table(ANN_results*1000, 'VariableNames', {'Resistance'});
Holtrop     = table(Holtrop_results*1000, 'VariableNames', {'Resistance'});
Hollenbach  = table(Hollenbach_results*1000, 'VariableNames', {'Resistance'});

clear ANN_results Holtrop_results Hollenbach_results Ann
%}

%% --------------- Create ANN for Cr ------------------
ANN_in  = zeros(9, FnSum);   % Dependent on number of input variables
ANN_out = zeros(1, FnSum);   % Dependemt on number of output variables

a   = 0;

for i = 1:n1
    for j = 1:maxS
        if Fn0(i,j) ~= 0
            a = a +1;
%            ANN_in(:,a)     = [Fn(i,j); Lwl(i)/Lpp(i); Lwl(i)/B(i); B(i)/T(i); (Tap(i)-Tfp(i))/Lpp(i); Cm(i); Geometry.Cb(i)];       % Parameterset #1
            ANN_in(:,a)     = [Fn(i,j); Lwl(i)/nthroot(DispV(i),3); Lwl(i)/Lpp(i); Lwl(i)/B(i); B(i)/T(i); (Tap(i)-Tfp(i))/Lpp(i); Geometry.AT(i)/(B(i)*T(i)); Cm(i); Geometry.Cb(i)];          % Parameter set #2
%            ANN_in(:,a)     = [Fn(i,j); Lwl(i)/B(i); B(i)/T(i); Geometry.Cb(i)];         % Parameter set #3

            ANN_out(a)      = Cr(i,j);
        end
    end
end

run ANN_Cr 

a   = 0;
test    = zeros(n1,maxS);
for i = 1:n1
    for j = 1:maxS
        if Fn0(i,j) ~= 0
            a = a +1;
            test(i,j)     = y(a);
        end
    end
end

TEST        = test ~= 0;
test        = test ./ TEST;


%ANN.test     = test;

ANN         = table(test, 'VariableNames', {'C_R'});


clear t Fn0 FnSum a test TEST ANN_in ANN_out

%% --------------- Mean Error and Standard Deviation [Ex] ------------------
%{
% Make working vectors for this section
RTS0    = Resistance.Total_Resistance;
Ann     = ANN.test;     %ANN.Resistance;
Di      = Digernes.Resistance;
Holl    = Hollenbach.Resistance;
Holt    = Holtrop.Resistance;
Oort    = Oortmerssen.Resistance;

% Convert NaN to 1
RTS0(isnan(RTS0))   = 1;
Ann(isnan(Ann))     = 1;
Di(isnan(Di))       = 1;
Holl(isnan(Holl))   = 1;
Holt(isnan(Holt))   = 1;
Oort(isnan(Oort))   = 1;

% Holtrop and ANN did not calculate resistance for all speed, this for-loop
% goes around this problem
for i  =1:n1                        
    for j = 1:maxS
        if Ann(i,j) == 1
            Ann(i,j) = RTS0(i,j);
        end
        if Holt(i,j) == 1
            Holt(i,j) = RTS0(i,j);
        end

    end
end

% Calculate error in percent and standard deviation for each method and vessel.
% Velocities that is not tested, is left out of the calculation
for i = 1:n1
    ANN_error(i,:)  = (100*(Ann(i,:) - RTS0(i,:))./RTS0(i,:));
    ANN_mean(i,:)   = sum(ANN_error(i,:))./sum(ANN_error(i,:)~=0);
    ANN_Std(i,:)    = sqrt((1./(sum(ANN_error(i,:)~=0).*(sum(ANN_error(i,:)~=0)-1))).*(sum(ANN_error(i,:)~=0).*sum(ANN_error(i,:).^2) - sum(ANN_error(i,:)).^2));
    
    Di_error(i,:)   = 100*(Di(i,:) - RTS0(i,:))./RTS0(i,:);
    Di_mean(i,:)    = sum(Di_error(i,:))./sum(Di_error(i,:)~=0);
    Di_Std(i,:)     = sqrt((1./(sum(Di_error(i,:)~=0).*(sum(Di_error(i,:)~=0)-1))).*(sum(Di_error(i,:)~=0).*sum(Di_error(i,:).^2) - sum(Di_error(i,:)).^2));
    
    Holl_error(i,:) = 100*(Holl(i,:) - RTS0(i,:))./RTS0(i,:);
    Holl_mean(i,:)  = sum(Holl_error(i,:))./sum(Holl_error(i,:)~=0);
    Holl_Std(i,:)   = sqrt((1./(sum(Holl_error(i,:)~=0).*(sum(Holl_error(i,:)~=0)-1))).*(sum(Holl_error(i,:)~=0).*sum(Holl_error(i,:).^2) - sum(Holl_error(i,:)).^2));
    
    Holt_error(i,:) = (100*(Holt(i,:) - RTS0(i,:))./RTS0(i,:));
    Holt_mean(i,:)  = sum(Holt_error(i,:))./sum(Holt_error(i,:)~=0);
    Holt_Std(i,:)   = sqrt((1./(sum(Holt_error(i,:)~=0).*(sum(Holt_error(i,:)~=0)-1))).*(sum(Holt_error(i,:)~=0).*sum(Holt_error(i,:).^2) - sum(Holt_error(i,:)).^2));
    
    Oort_error(i,:) = 100*(Oort(i,:) - RTS0(i,:))./RTS0(i,:);
    Oort_mean(i,:)  = sum(Oort_error(i,:))./sum(Oort_error(i,:)~=0);
    Oort_Std(i,:)   = sqrt((1./(sum(Oort_error(i,:)~=0).*(sum(Oort_error(i,:)~=0)-1))).*(sum(Oort_error(i,:)~=0).*sum(Oort_error(i,:).^2) - sum(Oort_error(i,:)).^2));
end

% Correcting the results for the vessels that ANN and Holtrop couldn't
% calculate the resistance to
ANN_mean(isnan(ANN_mean))   = 0;
ANN_Std(isnan(ANN_Std))     = 0;
Holt_mean(isnan(Holt_mean)) = 0;
Holt_Std(isnan(Holt_Std))   = 0;

% Calculate the total mean error and standard deviation for the whole
% material
ANN_Tmean       = sum((ANN_mean))./sum(ANN_mean~=0);
ANN_Tstd        = sqrt((1./(sum(ANN_mean~=0).*(sum(ANN_mean~=0)-1))).*(sum(ANN_mean~=0).*sum(ANN_mean.^2) - sum(ANN_mean).^2));

Di_Tmean        = sum((Di_mean))./sum(Di_mean~=0);
Di_Tstd         = sqrt((1./(sum(Di_mean~=0).*(sum(Di_mean~=0)-1))).*(sum(Di_mean~=0).*sum(Di_mean.^2) - sum(Di_mean).^2));

Holl_Tmean      = sum((Holl_mean))./sum(Holl_mean~=0);
Holl_Tstd       = sqrt((1./(sum(Holl_mean~=0).*(sum(Holl_mean~=0)-1))).*(sum(Holl_mean~=0).*sum(Holl_mean.^2) - sum(Holl_mean).^2));

Holt_Tmean      = sum((Holt_mean))./sum(Holt_mean~=0);
Holt_Tstd       = sqrt((1./(sum(Holt_mean~=0).*(sum(Holt_mean~=0)-1))).*(sum(Holt_mean~=0).*sum(Holt_mean.^2) - sum(Holt_mean).^2));

Oort_Tmean      = sum((Oort_mean))./sum(Oort_mean~=0);
Oort_Tstd       = sqrt((1./(sum(Oort_mean~=0).*(sum(Oort_mean~=0)-1))).*(sum(Oort_mean~=0).*sum(Oort_mean.^2) - sum(Oort_mean).^2));

% Adding results to tables
ANN.All_error   = ANN_error;
ANN.Mean_error  = ANN_mean;
ANN.Std_dev     = ANN_Std;

Digernes.All_error  = Di_error;
Digernes.Mean_error = Di_mean;
Digernes.Std_dev    = Di_Std;

Hollenbach.All_error    = Holl_error;
Hollenbach.Mean_error   = Holl_mean;
Hollenbach.Std_dev      = Holl_Std;

Holtrop.All_error   = Holt_error;
Holtrop.Mean_error  = Holt_mean;
Holtrop.Std_dev     = Holt_Std;

Oortmerssen.All_error   = Oort_error;
Oortmerssen.Mean_error  = Oort_mean;
Oortmerssen.Std_dev     = Oort_Std;

Error   = table([ANN_Tmean; Di_Tmean; Holl_Tmean; Holt_Tmean; Oort_Tmean], [ANN_Tstd; Di_Tstd; Holl_Tstd; Holt_Tstd; Oort_Tstd], 'VariableNames', {'Mean', 'Std_dev'}, 'RowNames', {'ANN', 'Digernes', 'Hollenbach', 'Holtrop', 'Oortmerssen'});

% Removing temporarily variables
clear RTS0 Ann Di Holl Holt Oort ANN_error ANN_mean ANN_Std ANN_Tmean ANN_Tstd Di_error Di_mean Di_Std Di_Tmean Di_Tstd Holl_error Holl_mean Holl_Std Holl_Tmean Holl_Tstd Holt_error Holt_mean Holt_Std Holt_Tmean Holt_Tstd Oort_error Oort_mean Oort_Std Oort_Tmean Oort_Tstd
%}

%% --------------- Mean Error and Standard Deviation at design speed [Ex] ------------------
%{
% Assumes that designspeed is the second highest tested velocity
for i = 1:n1
    notrun  = sum(isnan(RTS(i,:)));
    
    ANN_design(i)   = ANN.All_error(i, maxS-(notrun+1) );
    Di_design(i)    = Digernes.All_error(i, maxS-(notrun+1) );
    Holl_design(i)  = Hollenbach.All_error(i, maxS-(notrun+1) );
    Holt_design(i)  = Holtrop.All_error(i, maxS-(notrun+1) );
    Oort_design(i)  = Oortmerssen.All_error(i, maxS-(notrun+1) );
    
    Design_speed(i) = V(i, maxS-(notrun+1) );
end
%%{
% Calculate the mean error for each method
ANN_error   = sum(abs(ANN_design))./sum(ANN_design~=0);
Di_error    = sum(abs(Di_design))./sum(Di_design~=0);
Holl_error  = sum(abs(Holl_design))./sum(Holl_design~=0);
Holt_error  = sum(abs(Holt_design))./sum(Holt_design~=0);
Oort_error  = sum(abs(Oort_design))./sum(Oort_design~=0);

% Adding the results to table
ANN.Design_error        = ANN_design';
Digernes.Design_error   = Di_design';
Hollenbach.Design_error = Holl_design';
Holtrop.Design_error    = Holt_design';
Oortmerssen.Design_error= Oort_design';

Error.Design_error = [ANN_error; Di_error; Holl_error; Holt_error; Oort_error];

% Removing temporarily variables
%}
clear notrun ANN_design ANN_error Di_design Di_error Holl_design Holl_error Holt_design Holt_error Oort_design Oort_error

%% --------------- Mean Error and Standard Deviation at Froude intervals [Ex] ------------------
%{
% Choose the Froude number intervals
Fn1     = Fn <= 0.2;
Fn2     = Fn > 0.2 & Fn <= 0.3;
Fn3     = Fn > 0.3;

% Extract the error results for the chosen Fn intervals
ANN.All_error_Fn1 = ANN.All_error.*Fn1; 
ANN.All_error_Fn2 = ANN.All_error.*Fn2;
ANN.All_error_Fn3 = ANN.All_error.*Fn3;

Digernes.All_error_Fn1 = Digernes.All_error.*Fn1;
Digernes.All_error_Fn2 = Digernes.All_error.*Fn2;
Digernes.All_error_Fn3 = Digernes.All_error.*Fn3;

Hollenbach.All_error_Fn1 = Hollenbach.All_error.*Fn1;
Hollenbach.All_error_Fn2 = Hollenbach.All_error.*Fn2;
Hollenbach.All_error_Fn3 = Hollenbach.All_error.*Fn3;

Holtrop.All_error_Fn1 = Holtrop.All_error.*Fn1;
Holtrop.All_error_Fn2 = Holtrop.All_error.*Fn2;
Holtrop.All_error_Fn3 = Holtrop.All_error.*Fn3;

Oortmerssen.All_error_Fn1 = Oortmerssen.All_error.*Fn1;
Oortmerssen.All_error_Fn2 = Oortmerssen.All_error.*Fn2;
Oortmerssen.All_error_Fn3 = Oortmerssen.All_error.*Fn3;

% Calculate the mean error and standard deviation for each vessel in the
% Fn intervals
for i = 1:n1
    ANN_mean1(i)    = sum(ANN.All_error_Fn1(i,:))./sum(ANN.All_error_Fn1(i,:)~=0);
    ANN_mean2(i)    = sum(ANN.All_error_Fn2(i,:))./sum(ANN.All_error_Fn2(i,:)~=0);
    ANN_mean3(i)    = sum(ANN.All_error_Fn3(i,:))./sum(ANN.All_error_Fn3(i,:)~=0);
    
    ANN_Std1(i)     = sqrt((1./(sum(ANN.All_error_Fn1(i,:)~=0).*(sum(ANN.All_error_Fn1(i,:)~=0)-1))).*(sum(ANN.All_error_Fn1(i,:)~=0).*sum(ANN.All_error_Fn1(i,:).^2) - sum(ANN.All_error_Fn1(i,:)).^2));
    ANN_Std2(i)     = sqrt((1./(sum(ANN.All_error_Fn2(i,:)~=0).*(sum(ANN.All_error_Fn2(i,:)~=0)-1))).*(sum(ANN.All_error_Fn2(i,:)~=0).*sum(ANN.All_error_Fn2(i,:).^2) - sum(ANN.All_error_Fn2(i,:)).^2));
    ANN_Std3(i)     = sqrt((1./(sum(ANN.All_error_Fn3(i,:)~=0).*(sum(ANN.All_error_Fn3(i,:)~=0)-1))).*(sum(ANN.All_error_Fn3(i,:)~=0).*sum(ANN.All_error_Fn3(i,:).^2) - sum(ANN.All_error_Fn3(i,:)).^2));
    
    Di_mean1(i)     = sum(Digernes.All_error_Fn1(i,:))./sum(Digernes.All_error_Fn1(i,:)~=0);
    Di_mean2(i)     = sum(Digernes.All_error_Fn2(i,:))./sum(Digernes.All_error_Fn2(i,:)~=0);
    Di_mean3(i)     = sum(Digernes.All_error_Fn3(i,:))./sum(Digernes.All_error_Fn3(i,:)~=0);
    
    Di_Std1(i)      = sqrt((1./(sum(Digernes.All_error_Fn1(i,:)~=0).*(sum(Digernes.All_error_Fn1(i,:)~=0)-1))).*(sum(Digernes.All_error_Fn1(i,:)~=0).*sum(Digernes.All_error_Fn1(i,:).^2) - sum(Digernes.All_error_Fn1(i,:)).^2));
    Di_Std2(i)      = sqrt((1./(sum(Digernes.All_error_Fn2(i,:)~=0).*(sum(Digernes.All_error_Fn2(i,:)~=0)-1))).*(sum(Digernes.All_error_Fn2(i,:)~=0).*sum(Digernes.All_error_Fn2(i,:).^2) - sum(Digernes.All_error_Fn2(i,:)).^2));
    Di_Std3(i)      = sqrt((1./(sum(Digernes.All_error_Fn3(i,:)~=0).*(sum(Digernes.All_error_Fn3(i,:)~=0)-1))).*(sum(Digernes.All_error_Fn3(i,:)~=0).*sum(Digernes.All_error_Fn3(i,:).^2) - sum(Digernes.All_error_Fn3(i,:)).^2));
    
    Holl_mean1(i)   = sum(Hollenbach.All_error_Fn1(i,:))./sum(Hollenbach.All_error_Fn1(i,:)~=0);
    Holl_mean2(i)   = sum(Hollenbach.All_error_Fn2(i,:))./sum(Hollenbach.All_error_Fn2(i,:)~=0);
    Holl_mean3(i)   = sum(Hollenbach.All_error_Fn3(i,:))./sum(Hollenbach.All_error_Fn3(i,:)~=0);
    
    Holl_Std1(i)    = sqrt((1./(sum(Hollenbach.All_error_Fn1(i,:)~=0).*(sum(Hollenbach.All_error_Fn1(i,:)~=0)-1))).*(sum(Hollenbach.All_error_Fn1(i,:)~=0).*sum(Hollenbach.All_error_Fn1(i,:).^2) - sum(Hollenbach.All_error_Fn1(i,:)).^2));
    Holl_Std2(i)    = sqrt((1./(sum(Hollenbach.All_error_Fn2(i,:)~=0).*(sum(Hollenbach.All_error_Fn2(i,:)~=0)-1))).*(sum(Hollenbach.All_error_Fn2(i,:)~=0).*sum(Hollenbach.All_error_Fn2(i,:).^2) - sum(Hollenbach.All_error_Fn2(i,:)).^2));
    Holl_Std3(i)    = sqrt((1./(sum(Hollenbach.All_error_Fn3(i,:)~=0).*(sum(Hollenbach.All_error_Fn3(i,:)~=0)-1))).*(sum(Hollenbach.All_error_Fn3(i,:)~=0).*sum(Hollenbach.All_error_Fn3(i,:).^2) - sum(Hollenbach.All_error_Fn3(i,:)).^2));
        
    Holt_mean1(i)   = sum(Holtrop.All_error_Fn1(i,:))./sum(Holtrop.All_error_Fn1(i,:)~=0);
    Holt_mean2(i)   = sum(Holtrop.All_error_Fn2(i,:))./sum(Holtrop.All_error_Fn2(i,:)~=0);
    Holt_mean3(i)   = sum(Holtrop.All_error_Fn3(i,:))./sum(Holtrop.All_error_Fn3(i,:)~=0);
    
    Holt_Std1(i)    = sqrt((1./(sum(Holtrop.All_error_Fn1(i,:)~=0).*(sum(Holtrop.All_error_Fn1(i,:)~=0)-1))).*(sum(Holtrop.All_error_Fn1(i,:)~=0).*sum(Holtrop.All_error_Fn1(i,:).^2) - sum(Holtrop.All_error_Fn1(i,:)).^2));
    Holt_Std2(i)    = sqrt((1./(sum(Holtrop.All_error_Fn2(i,:)~=0).*(sum(Holtrop.All_error_Fn2(i,:)~=0)-1))).*(sum(Holtrop.All_error_Fn2(i,:)~=0).*sum(Holtrop.All_error_Fn2(i,:).^2) - sum(Holtrop.All_error_Fn2(i,:)).^2));
    Holt_Std3(i)    = sqrt((1./(sum(Holtrop.All_error_Fn3(i,:)~=0).*(sum(Holtrop.All_error_Fn3(i,:)~=0)-1))).*(sum(Holtrop.All_error_Fn3(i,:)~=0).*sum(Holtrop.All_error_Fn3(i,:).^2) - sum(Holtrop.All_error_Fn3(i,:)).^2));

    Oort_mean1(i)   = sum(Oortmerssen.All_error_Fn1(i,:))./sum(Oortmerssen.All_error_Fn1(i,:)~=0);
    Oort_mean2(i)   = sum(Oortmerssen.All_error_Fn2(i,:))./sum(Oortmerssen.All_error_Fn2(i,:)~=0);
    Oort_mean3(i)   = sum(Oortmerssen.All_error_Fn3(i,:))./sum(Oortmerssen.All_error_Fn3(i,:)~=0);
    
    Oort_Std1(i)    = sqrt((1./(sum(Oortmerssen.All_error_Fn1(i,:)~=0).*(sum(Oortmerssen.All_error_Fn1(i,:)~=0)-1))).*(sum(Oortmerssen.All_error_Fn1(i,:)~=0).*sum(Oortmerssen.All_error_Fn1(i,:).^2) - sum(Oortmerssen.All_error_Fn1(i,:)).^2));
    Oort_Std2(i)    = sqrt((1./(sum(Oortmerssen.All_error_Fn2(i,:)~=0).*(sum(Oortmerssen.All_error_Fn2(i,:)~=0)-1))).*(sum(Oortmerssen.All_error_Fn2(i,:)~=0).*sum(Oortmerssen.All_error_Fn2(i,:).^2) - sum(Oortmerssen.All_error_Fn2(i,:)).^2));
    Oort_Std3(i)    = sqrt((1./(sum(Oortmerssen.All_error_Fn3(i,:)~=0).*(sum(Oortmerssen.All_error_Fn3(i,:)~=0)-1))).*(sum(Oortmerssen.All_error_Fn3(i,:)~=0).*sum(Oortmerssen.All_error_Fn3(i,:).^2) - sum(Oortmerssen.All_error_Fn3(i,:)).^2));
end

% Convert NaN values to zeros
ANN_mean1(isnan(ANN_mean1))     = 0;
ANN_mean2(isnan(ANN_mean2))     = 0;
ANN_mean3(isnan(ANN_mean3))     = 0;
ANN_Std1(isnan(ANN_Std1))       = 0;
ANN_Std2(isnan(ANN_Std2))       = 0;
ANN_Std3(isnan(ANN_Std3))       = 0;

Di_mean1(isnan(Di_mean1))       = 0;
Di_mean2(isnan(Di_mean2))       = 0;
Di_mean3(isnan(Di_mean3))       = 0;
Di_Std1(isnan(Di_mean1))        = 0;
Di_Std2(isnan(Di_mean2))        = 0;
Di_Std3(isnan(Di_mean3))        = 0;

Holl_mean1(isnan(Holl_mean1))   = 0;
Holl_mean2(isnan(Holl_mean2))   = 0;
Holl_mean3(isnan(Holl_mean3))   = 0;
Holl_Std1(isnan(Holl_Std1))     = 0;
Holl_Std2(isnan(Holl_Std2))     = 0;
Holl_Std3(isnan(Holl_Std3))     = 0;

Holt_mean1(isnan(Holt_mean1))   = 0;
Holt_mean2(isnan(Holt_mean2))   = 0;
Holt_mean3(isnan(Holt_mean3))   = 0;
Holt_Std1(isnan(Holt_Std1))     = 0;
Holt_Std2(isnan(Holt_Std2))     = 0;
Holt_Std3(isnan(Holt_Std3))     = 0;

Oort_mean1(isnan(Oort_mean1))   = 0;
Oort_mean2(isnan(Oort_mean2))   = 0;
Oort_mean3(isnan(Oort_mean3))   = 0;
Oort_Std1(isnan(Oort_Std1))     = 0;
Oort_Std2(isnan(Oort_Std2))     = 0;
Oort_Std3(isnan(Oort_Std3))     = 0;

% Calculating total mean error and standard deviation
ANN_Tmean1  = sum((ANN_mean1))./sum(ANN_mean1~=0); 
ANN_Tmean2  = sum((ANN_mean2))./sum(ANN_mean2~=0);
ANN_Tmean3  = sum((ANN_mean3))./sum(ANN_mean3~=0);
ANN_Tstd1   = sqrt((1./(sum(ANN_mean1~=0).*(sum(ANN_mean1~=0)-1))).*(sum(ANN_mean1~=0).*sum(ANN_mean1.^2) - sum(ANN_mean1).^2));
ANN_Tstd2   = sqrt((1./(sum(ANN_mean2~=0).*(sum(ANN_mean2~=0)-1))).*(sum(ANN_mean2~=0).*sum(ANN_mean2.^2) - sum(ANN_mean2).^2));
ANN_Tstd3   = sqrt((1./(sum(ANN_mean3~=0).*(sum(ANN_mean3~=0)-1))).*(sum(ANN_mean3~=0).*sum(ANN_mean3.^2) - sum(ANN_mean3).^2));

Di_Tmean1   = sum((Di_mean1))./sum(Di_mean1~=0); 
Di_Tmean2   = sum((Di_mean2))./sum(Di_mean2~=0);
Di_Tmean3   = sum((Di_mean3))./sum(Di_mean3~=0);
Di_Tstd1    = sqrt((1./(sum(Di_mean1~=0).*(sum(Di_mean1~=0)-1))).*(sum(Di_mean1~=0).*sum(Di_mean1.^2) - sum(Di_mean1).^2));
Di_Tstd2    = sqrt((1./(sum(Di_mean2~=0).*(sum(Di_mean2~=0)-1))).*(sum(Di_mean2~=0).*sum(Di_mean2.^2) - sum(Di_mean2).^2));
Di_Tstd3    = sqrt((1./(sum(Di_mean3~=0).*(sum(Di_mean3~=0)-1))).*(sum(Di_mean3~=0).*sum(Di_mean3.^2) - sum(Di_mean3).^2));

Holl_Tmean1 = sum((Holl_mean1))./sum(Holl_mean1~=0); 
Holl_Tmean2 = sum((Holl_mean2))./sum(Holl_mean2~=0);
Holl_Tmean3 = sum((Holl_mean3))./sum(Holl_mean3~=0);
Holl_Tstd1  = sqrt((1./(sum(Holl_mean1~=0).*(sum(Holl_mean1~=0)-1))).*(sum(Holl_mean1~=0).*sum(Holl_mean1.^2) - sum(Holl_mean1).^2));
Holl_Tstd2  = sqrt((1./(sum(Holl_mean2~=0).*(sum(Holl_mean2~=0)-1))).*(sum(Holl_mean2~=0).*sum(Holl_mean2.^2) - sum(Holl_mean2).^2));
Holl_Tstd3  = sqrt((1./(sum(Holl_mean3~=0).*(sum(Holl_mean3~=0)-1))).*(sum(Holl_mean3~=0).*sum(Holl_mean3.^2) - sum(Holl_mean3).^2));

Holt_Tmean1 = sum((Holt_mean1))./sum(Holt_mean1~=0); 
Holt_Tmean2 = sum((Holt_mean2))./sum(Holt_mean2~=0);
Holt_Tmean3 = sum((Holt_mean3))./sum(Holt_mean3~=0);
Holt_Tstd1  = sqrt((1./(sum(Holt_mean1~=0).*(sum(Holt_mean1~=0)-1))).*(sum(Holt_mean1~=0).*sum(Holt_mean1.^2) - sum(Holt_mean1).^2));
Holt_Tstd2  = sqrt((1./(sum(Holt_mean2~=0).*(sum(Holt_mean2~=0)-1))).*(sum(Holt_mean2~=0).*sum(Holt_mean2.^2) - sum(Holt_mean2).^2));
Holt_Tstd3  = sqrt((1./(sum(Holt_mean3~=0).*(sum(Holt_mean3~=0)-1))).*(sum(Holt_mean3~=0).*sum(Holt_mean3.^2) - sum(Holt_mean3).^2));

Oort_Tmean1 = sum((Oort_mean1))./sum(Oort_mean1~=0); 
Oort_Tmean2 = sum((Oort_mean2))./sum(Oort_mean2~=0);
Oort_Tmean3 = sum((Oort_mean3))./sum(Oort_mean3~=0);
Oort_Tstd1  = sqrt((1./(sum(Oort_mean1~=0).*(sum(Oort_mean1~=0)-1))).*(sum(Oort_mean1~=0).*sum(Oort_mean1.^2) - sum(Oort_mean1).^2));
Oort_Tstd2  = sqrt((1./(sum(Oort_mean2~=0).*(sum(Oort_mean2~=0)-1))).*(sum(Oort_mean2~=0).*sum(Oort_mean2.^2) - sum(Oort_mean2).^2));
Oort_Tstd3  = sqrt((1./(sum(Oort_mean3~=0).*(sum(Oort_mean3~=0)-1))).*(sum(Oort_mean3~=0).*sum(Oort_mean3.^2) - sum(Oort_mean3).^2));

% Adding the results to tables
Error.Mean_Fn1      = [ANN_Tmean1; Di_Tmean1; Holl_Tmean1; Holt_Tmean1; Oort_Tmean1];
Error.Mean_Fn2      = [ANN_Tmean2; Di_Tmean2; Holl_Tmean2; Holt_Tmean2; Oort_Tmean2];
Error.Mean_Fn3      = [ANN_Tmean3; Di_Tmean3; Holl_Tmean3; Holt_Tmean3; Oort_Tmean3];

Error.Std_dev_Fn1   = [ANN_Tstd1; Di_Tstd1; Holl_Tstd1; Holt_Tstd1; Oort_Tstd1];
Error.Std_dev_Fn2   = [ANN_Tstd2; Di_Tstd2; Holl_Tstd2; Holt_Tstd2; Oort_Tstd2];
Error.Std_dev_Fn3   = [ANN_Tstd3; Di_Tstd3; Holl_Tstd3; Holt_Tstd3; Oort_Tstd3];

ANN.Mean_error_Fn1  = ANN_mean1';
ANN.Mean_error_Fn2  = ANN_mean2';
ANN.Mean_error_Fn3  = ANN_mean3';
ANN.Std_dev_Fn1     = ANN_Std1';
ANN.Std_dev_Fn2     = ANN_Std2';
ANN.Std_dev_Fn3     = ANN_Std3';

Digernes.Mean_error_Fn1  = Di_mean1';
Digernes.Mean_error_Fn2  = Di_mean2';
Digernes.Mean_error_Fn3  = Di_mean3';
Digernes.Std_dev_Fn1     = Di_Std1';
Digernes.Std_dev_Fn2     = Di_Std2';
Digernes.Std_dev_Fn3     = Di_Std3';

Hollenbach.Mean_error_Fn1  = Holl_mean1';
Hollenbach.Mean_error_Fn2  = Holl_mean2';
Hollenbach.Mean_error_Fn3  = Holl_mean3';
Hollenbach.Std_dev_Fn1     = Holl_Std1';
Hollenbach.Std_dev_Fn2     = Holl_Std2';
Hollenbach.Std_dev_Fn3     = Holl_Std3';

Holtrop.Mean_error_Fn1  = Holt_mean1';
Holtrop.Mean_error_Fn2  = Holt_mean2';
Holtrop.Mean_error_Fn3  = Holt_mean3';
Holtrop.Std_dev_Fn1     = Holt_Std1';
Holtrop.Std_dev_Fn2     = Holt_Std2';
Holtrop.Std_dev_Fn3     = Holt_Std3';

Oortmerssen.Mean_error_Fn1  = Oort_mean1';
Oortmerssen.Mean_error_Fn2  = Oort_mean2';
Oortmerssen.Mean_error_Fn3  = Oort_mean3';
Oortmerssen.Std_dev_Fn1     = Oort_Std1';
Oortmerssen.Std_dev_Fn2     = Oort_Std2';
Oortmerssen.Std_dev_Fn3     = Oort_Std3';

% Removing the temporarily variables
clear   ANN_mean1 ANN_mean2 ANN_mean3 ANN_Std1 ANN_Std2 ANN_Std3 Di_mean1 Di_mean2 Di_mean3 Di_Std1 Di_Std2 Di_Std3 ...
        Holl_mean1 Holl_mean2 Holl_mean3 Holl_Std1 Holl_Std2 Holl_Std3 Holt_mean1 Holt_mean2 Holt_mean3 Holt_Std1 Holt_Std2 Holt_Std3 ...
        Oort_mean1 Oort_mean2 Oort_mean3 Oort_Std1 Oort_Std2 Oort_Std3 ...
        ANN_Tmean1 ANN_Tmean2 ANN_Tmean3 ANN_Tstd1 ANN_Tstd2 ANN_Tstd3 Di_Tmean1 Di_Tmean2 Di_Tmean3 Di_Tstd1 Di_Tstd2 Di_Tstd3 ...
        Holl_Tmean1 Holl_Tmean2 Holl_Tmean3 Holl_Tstd1 Holl_Tstd2 Holl_Tstd3 Holt_Tmean1 Holt_Tmean2 Holt_Tmean3 Holt_Tstd1 Holt_Tstd2 Holt_Tstd3 ...
        Oort_Tmean1 Oort_Tmean2 Oort_Tmean3 Oort_Tstd1 Oort_Tstd2 Oort_Tstd3
%}

%% --------------- Writing results to .txt files [Ex] ------------------
%{
% Header for Error file
fprintf(Error_file, 'Mean Error ANN %8.4f \n', Error.Mean(1));
fprintf(Error_file, 'Mean Error Digernes %8.4f \n', Error.Mean(2));
fprintf(Error_file, 'Mean Error Hollenbach %8.4f \n', Error.Mean(3));
fprintf(Error_file, 'Mean Error Holtrop %8.4f \n', Error.Mean(4));
fprintf(Error_file, 'Mean Error Oortmerssen %8.4f \n \n', Error.Mean(5));

fprintf(Error_file, 'Mean Standard deviation ANN %8.4f \n', Error.Std_dev(1));
fprintf(Error_file, 'Mean Standard deviation Digernes %8.4f \n', Error.Std_dev(2));
fprintf(Error_file, 'Mean Standard deviation Hollenbach %8.4f \n', Error.Std_dev(3));
fprintf(Error_file, 'Mean Standard deviation Holtrop %8.4f \n', Error.Std_dev(4));
fprintf(Error_file, 'Mean Standard deviation Oortmerssen %8.4f \n \n', Error.Std_dev(5));

fprintf(Error_file, '%10s %5s %5s %13s %13s %13s %13s %13s %13s %13s %13s %13s %13s','Model no.', 'Ver.', 'WL', 'M. error ANN', 'M. error Di', 'M. error Holl', 'M. error Holt', 'M. error Oort', 'Std dev ANN',  'Std dev Di', 'Std dev Holl', 'Std dev Holt', 'Std dev Oort');
fprintf(Error_file, '\n');

% Header for Error_design file
fprintf(Error_design_file, 'Mean error at estimated design speed ANN %10.4f \n \n', Error.Design_error(1));
fprintf(Error_design_file, 'Mean error at estimated design speed Digernes %10.4f \n', Error.Design_error(2));
fprintf(Error_design_file, 'Mean error at estimated design speed Hollenbach %10.4f \n', Error.Design_error(3));
fprintf(Error_design_file, 'Mean error at estimated design speed Holtrop %10.4f \n', Error.Design_error(4));
fprintf(Error_design_file, 'Mean error at estimated design speed Oortmerssen %10.4f \n \n', Error.Design_error(5));

fprintf(Error_design_file, '%10s %5s %5s %19s %13s %13s %13s %13s %13s', 'Model no.', 'Ver.', 'WL', 'Estimated d.speed', 'Error ANN', 'Error Di', 'Error Holl', 'Error Holt', 'Error Oort');
fprintf(Error_design_file, '\n');

% Header for Error_Fn file
fprintf(Error_Fn_file,'Mean Error ANN %8.4f \n', Error.Mean_Fn1(1));
fprintf(Error_Fn_file,'Mean Error Digernes %8.4f \n', Error.Mean_Fn1(2));
fprintf(Error_Fn_file,'Mean Error Hollenbach %8.4f \n', Error.Mean_Fn1(3));
fprintf(Error_Fn_file,'Mean Error Holtrop %8.4f \n', Error.Mean_Fn1(4));
fprintf(Error_Fn_file,'Mean Error Oortmerssen %8.4f \n \n', Error.Mean_Fn1(5));

fprintf(Error_Fn_file,'%10s & %13s & %13s & %13s & %13s & %13s \\\\','Number', 'ANN', 'Digernes', 'Hollenbach', 'Holtrop', 'Oortmerssen');
fprintf(Error_Fn_file,'\n');

% Header for Std_dev_Fn file
fprintf(Std_dev_Fn_file,'StD ANN %8.4f \n', Error.Std_dev_Fn1(1));
fprintf(Std_dev_Fn_file,'StD Digernes %8.4f \n', Error.Std_dev_Fn1(2));
fprintf(Std_dev_Fn_file,'StD Hollenbach %8.4f \n', Error.Std_dev_Fn1(3));
fprintf(Std_dev_Fn_file,'StD Holtrop %8.4f \n', Error.Std_dev_Fn1(4));
fprintf(Std_dev_Fn_file,'StD Oortmerssen %8.4f \n \n', Error.Std_dev_Fn1(5));

fprintf(Std_dev_Fn_file,'%10s & %13s & %13s & %13s & %13s & %13s \\\\','Number', 'ANN', 'Digernes', 'Hollenbach', 'Holtrop', 'Oortmerssen');
fprintf(Std_dev_Fn_file,'\n');

% Header for Vessel overview file
fprintf(Vessel_overview_file,'%10s & %13s & %13s & %13s & %13s & %13s & %13s & %10s & %5s & %5s\\\\','Number', 'B/T', 'Lwl/B', 'Fn_min', 'Fn_max', 'C_B', 'Ant. Prop', 'Model no.', 'Ver.', 'WL');
fprintf(Vessel_overview_file,'\n');

for i = 1:n1
    % Fill in results for Error file
    fprintf(Error_file, '%10i %5i %5i %13.4f %13.4f %13.4f %13.4f %13.4f %13.4f %13.4f %13.4f %13.4f %13.4f \n', ...
    Geometry.Model(i), Geometry.Ver(i), Geometry.WL(i), ...
    ANN.Mean_error(i), Digernes.Mean_error(i), Hollenbach.Mean_error(i), Holtrop.Mean_error(i), Oortmerssen.Mean_error(i), ...
    ANN.Std_dev(i), Digernes.Std_dev(i), Hollenbach.Std_dev(i), Holtrop.Std_dev(i), Oortmerssen.Std_dev(i));
    
    % Fill inn results for Error_design file
    fprintf(Error_design_file, '%10i %5i %5i %19.2f %13.4f %13.4f %13.4f %13.4f %13.4f \n', ...
    Geometry.Model(i), Geometry.Ver(i), Geometry.WL(i), Design_speed(i), ...
    ANN.Design_error(i), Digernes.Design_error(i), Hollenbach.Design_error(i), Holtrop.Design_error(i), Oortmerssen.Design_error(i));
    
    % Fill in results for Error_Fn file (Made for Latex)
    fprintf(Error_Fn_file,'%10i & %13.4f & %13.4f & %13.4f & %13.4f & %13.4f \\\\ \n', ...
    i, ANN.Mean_error_Fn1(i), Digernes.Mean_error_Fn1(i), Hollenbach.Mean_error_Fn1(i), Holtrop.Mean_error_Fn1(i), Oortmerssen.Mean_error_Fn1(i));
    
    % Fill in results for Std_dev_Fn file (Made for Latex)
    fprintf(Std_dev_Fn_file,'%10i & %13.4f & %13.4f & %13.4f & %13.4f & %13.4f  \\\\ \n', ...
    i, ANN.Std_dev_Fn1(i), Digernes.Std_dev_Fn1(i), Hollenbach.Std_dev_Fn1(i), Holtrop.Std_dev_Fn1(i), Oortmerssen.Std_dev_Fn1(i));
    
    % Fill in results for Vessel_overview file (Made for Latex)
    fprintf(Vessel_overview_file,'%10i & %13.4f & %13.4f & %13.4f & %13.4f & %13.4f & %14i & %10i & %5i & %5i \\\\ \n', ... 
    i, B(i)/T(i), Lwl(i)/T(i), min(Fn(i,:)), max(Fn(i,:)), Geometry.Cb(i), AntP(i), Geometry.Model(i), Geometry.Ver(i), Geometry.WL(i));
end
%}

%% --------------- Interpolate propulsion data ------------------
for i = 1:n1
    t_temp  = Propulsion.t(i,:);
    w_temp  = Propulsion.wake(i,:);
    
    t_temp (isnan(t_temp ))     = [];
    w_temp (isnan(w_temp ))     = [];
    
    t_mean(i)   = mean(t_temp);
    
    w_mean(i)   = mean(w_temp);
end

%{
for i = 1:n1
    notrun  = sum(isnan(RTS(i,:)));
    
    Design_speed(i) = Fn(i, maxS-(notrun+1) );
end

% Create vector without NaN values for each vessel, to estimate design
% values
for i = 1:n1
    t_temp  = Propulsion.t(i,:);
    w_temp  = Propulsion.wake(i,:);
    Fn_temp = Propulsion.Fn(i,:);
    
    t_temp (isnan(t_temp ))     = [];
    w_temp (isnan(w_temp ))     = [];
    Fn_temp(isnan(Fn_temp))     = [];
    
    t_mean(i)   = mean(t_temp);
    t_Lin       = fit(Fn_temp', t_temp', 'poly1');
    t_Sec       = fit(Fn_temp', t_temp', 'poly2');
    
    w_mean(i)   = mean(w_temp);
    w_Lin       = fit(Fn_temp', w_temp', 'poly1');
    w_Sec       = fit(Fn_temp', w_temp', 'poly2');
    
    t_lin(i)    = t_Lin(Design_speed(i));
    t_sec(i)    = t_Sec(Design_speed(i));
    
    w_lin(i)    = w_Lin(Design_speed(i));
    w_sec(i)    = w_Sec(Design_speed(i));
end
%}
% Conclusion:
% Mean value is good, and the most stable estimate. Also for designspeed

clear t_temp t_mean w_temp w_mean

%% --------------- Clean propulsion data for ANN ------------------

% Here, the plan is to remove inf, NaN and zero values in the dataset to
% avoid any disturbances in the final methods

%% --------------- Create ANN for thrustdeduction ------------------
t0                  = Propulsion.t;   
t0(isnan(t0))       = 0;
t0(t0~=0)           = 1;
tSum                = sum(sum(t0));



ANN_in  = zeros(5, tSum);   % Dependent on number of input variables
ANN_out = zeros(1, tSum);   % Dependemt on number of output variables

a   = 0;

for i = 1:n1
 	if t0(i) ~= 0
        a = a +1;
        ANN_in(:,a)     = [AntP(i); Geometry.Cb(i); LCB(i); B(i)/T(i); Lwl(i)/B(i)];       % Parameterset #1
%       ANN_in(:,a)     = [AntP(i); T_B];          % Parameter set #2
%       ANN_in(:,a)     = [];         % Parameter set #3

        ANN_out(a)      = Propulsion.t(i);
	end
end




run ANN_t

a   = 0;
test    = zeros(n1,maxS);
for i = 1:n1
  	if t0 ~= 0
      	a = a +1;
       	test(i)     = y(a);
   	end
end

TEST        = test ~= 0;
test        = test ./ TEST;


%ANN.test     = test;

ANN         = table(test, 'VariableNames', {'t'});


clear t Fn0 FnSum a test TEST ANN_in ANN_out

%% --------------- Close Files ------------------
fclose(Error_file);
fclose(Error_design_file);
fclose(Error_Fn_file);
fclose(Std_dev_Fn_file);
fclose(Vessel_overview_file);

clear Error_file Error_design_file Error_Fn_file Std_dev_Fn_file Vessel_overview_file

%% --------------- Plot Resistance curves [Ex] ------------------
% gives the possibility to prevent the resistance curves to show
%{
%for i = 1:n1 
  
    % Plot results from tested methods
    figure(99)
    % Plot model results
    plot(Fn', Cr', 'LineWidth',2)
    colormap(autumn)

    % Set up plot
    xlabel('F_N')
    ylabel('C_R')
    title('99');

%end
%}
%{ 
for i = 1:n1 
    j = 100+i;
    
    tittel = num2str(i);
    
    % Plot results from tested methods
    figure(j)
    plot(Fn(i,:), ANN.Resistance(i,:), '--mp', Fn(i,:), Digernes.Resistance(i,:), '-.gs', Fn(i,:), Hollenbach.Resistance(i,:), '--r^', Fn(i,:), Holtrop.Resistance(i,:), '-.bo', Fn(i,:), Oortmerssen.Resistance(i,:), '--kd', 'LineWidth',2)
    hold on
    % Plot model results
    plot(Fn(i,:), RTS(i,:),'Color',[1 0.637 0.273],'LineWidth',2)
    hold off
    
    % Set up plot
    xlabel('F_N')
    ylabel('R_T [N]')
    title(tittel);
    legend('ANN', 'Digernes', 'Hollenbach', 'Holtrop', 'Oortmerssen', 'Modell', 'Location', 'NorthWest')
    
end
%}

%% --------------- Plot Error [Ex] ------------------
% %{ gives the possibility to prevent the error curves to show
%{

% Error at designspeed
figure(201)
line([min(Lwl./B), max(Lwl./B)], [0, 0],'LineWidth',1)
hold on
plot(Lwl./B, ANN.Design_error, 'mp', Lwl./B, Digernes.Design_error, 'gs', Lwl./B, Hollenbach.Design_error, 'r^', Lwl./B, Holtrop.Design_error, 'bo', Lwl./B, Oortmerssen.Design_error, 'kd')
xlabel('^L/_B', 'FontSize', 14)
ylabel('Avvik [%]')
legend1 = legend('0% Error', 'Des. error ANN', 'Des. error Digernes', 'Des. error Hollenbach', 'Des. error Holtrop', 'Des. error Oortmerssen');
set(legend1, 'FontSize', 14);
hold off

figure(202)
line([min(DispV), max(DispV)], [0, 0],'LineWidth',1)
hold on
plot(DispV, ANN.Design_error, 'mp', DispV, Digernes.Design_error, 'gs', DispV, Hollenbach.Design_error, 'r^', DispV, Holtrop.Design_error, 'bo', DispV, Oortmerssen.Design_error, 'kd')
xlabel('Volumedisplacement [m^3]', 'FontSize', 14)
ylabel('Avvik [%]')
legend1 = legend('0% Error', 'Des. error ANN', 'Des. error Digernes', 'Des. error Hollenbach', 'Des. error Holtrop', 'Des. error Oortmerssen');
set(legend1, 'FontSize', 14);
hold off

figure(203)
line([min(Geometry.Cp), max(Geometry.Cp)], [0, 0],'LineWidth',1)
hold on
plot(Geometry.Cp, ANN.Design_error, 'mp', Geometry.Cp, Digernes.Design_error, 'gs', Geometry.Cp, Hollenbach.Design_error, 'r^', Geometry.Cp, Holtrop.Design_error, 'bo', Geometry.Cp, Oortmerssen.Design_error, 'kd')
xlabel('C_P', 'FontSize', 14)
ylabel('Avvik [%]')
legend1 = legend('0% Error', 'Des. error ANN', 'Des. error Digernes', 'Des. error Hollenbach', 'Des. error Holtrop', 'Des. error Oortmerssen');
set(legend1, 'FontSize', 14);
hold off

figure(204)
line([min(B./T), max(B./T)], [0, 0],'LineWidth',1)
hold on
plot(B./T, ANN.Design_error, 'mp', B./T, Digernes.Design_error, 'gs', B./T, Hollenbach.Design_error, 'r^', B./T, Holtrop.Design_error, 'bo', B./T, Oortmerssen.Design_error, 'kd')
xlabel('^B/_T', 'FontSize', 14)
ylabel('Avvik [%]')
legend1 = legend('0% Error', 'Des. error ANN', 'Des. error Digernes', 'Des. error Hollenbach', 'Des. error Holtrop', 'Des. error Oortmerssen');
set(legend1, 'FontSize', 14);
hold off

figure(205)
line([min(Lwl), max(Lwl)], [0, 0],'LineWidth',1)
hold on
plot(Lwl, ANN.Design_error, 'mp', Lwl, Digernes.Design_error, 'gs', Lwl, Hollenbach.Design_error, 'r^', Lwl, Holtrop.Design_error, 'bo', Lwl, Oortmerssen.Design_error, 'kd')
xlabel('L_W_L [m]', 'FontSize', 14)
ylabel('Avvik [%]')
legend1 = legend('0% Error', 'Des. error ANN', 'Des. error Digernes', 'Des. error Hollenbach', 'Des. error Holtrop', 'Des. error Oortmerssen');
set(legend1, 'FontSize', 14);
hold off

figure(206)
line([min(Tap-Tfp), max(Tap-Tfp)], [0, 0],'LineWidth',1)
hold on
plot(Tap-Tfp, ANN.Design_error, 'mp', Tap-Tfp, Digernes.Design_error, 'gs', Tap-Tfp, Hollenbach.Design_error, 'r^', Tap-Tfp, Holtrop.Design_error, 'bo', Tap-Tfp, Oortmerssen.Design_error, 'kd')
xlabel('Designtrim [m]', 'FontSize', 14)
ylabel('Avvik [%]')
legend1 = legend('0% Error', 'Des. error ANN', 'Des. error Digernes', 'Des. error Hollenbach', 'Des. error Holtrop', 'Des. error Oortmerssen');
set(legend1, 'FontSize', 14);
hold off

figure(207)
line([min(Geometry.Cb), max(Geometry.Cb)], [0, 0],'LineWidth',1)
hold on
plot(Geometry.Cb, ANN.Design_error, 'mp', Geometry.Cb, Digernes.Design_error, 'gs', Geometry.Cb, Hollenbach.Design_error, 'r^', Geometry.Cb, Holtrop.Design_error, 'bo', Geometry.Cb, Oortmerssen.Design_error, 'kd')
xlabel('C_B', 'FontSize', 14)
ylabel('Avvik [%]')
legend1 = legend('0% Error', 'Des. error ANN', 'Des. error Digernes', 'Des. error Hollenbach', 'Des. error Holtrop', 'Des. error Oortmerssen');
set(legend1, 'FontSize', 14);
hold off

figure(208)
line([1, n1], [0, 0],'LineWidth',1)
hold on
plot(1:n1, ANN.Design_error, 'mp', 1:n1, Digernes.Design_error, 'gs', 1:n1, Hollenbach.Design_error, 'r^', 1:n1, Holtrop.Design_error, 'bo', 1:n1, Oortmerssen.Design_error, 'kd')
xlabel('Serial number - kronologisk', 'FontSize', 14)
ylabel('Avvik [%]')
legend1 = legend('0% Error', 'Des. error ANN', 'Des. error Digernes', 'Des. error Hollenbach', 'Des. error Holtrop', 'Des. error Oortmerssen');
set(legend1, 'FontSize', 14);
hold off

% Mean Error
figure(301)
line([min(Lwl./B), max(Lwl./B)], [0, 0],'LineWidth',1)
hold on
plot(Lwl./B, ANN.Mean_error, 'mp', Lwl./B, Digernes.Mean_error, 'gs', Lwl./B, Hollenbach.Mean_error, 'r^', Lwl./B, Holtrop.Mean_error, 'bo', Lwl./B, Oortmerssen.Mean_error, 'kd')
xlabel('^L/_B', 'FontSize', 14)
ylabel('Avvik [%]')
legend1 = legend('0% Error', 'M. error ANN', 'M. error Digernes', 'M. error Hollenbach', 'M. error Holtrop', 'M. error Oortmerssen');
set(legend1, 'FontSize', 14);
hold off

figure(302)
line([min(DispV), max(DispV)], [0, 0],'LineWidth',1)
hold on
plot(DispV, ANN.Mean_error, 'mp', DispV, Digernes.Mean_error, 'gs', DispV, Hollenbach.Mean_error, 'r^', DispV, Holtrop.Mean_error, 'bo', DispV, Oortmerssen.Mean_error, 'kd')
xlabel('Volumedisplacement [m^3]', 'FontSize', 14)
ylabel('Avvik [%]')
legend1 = legend('0% Error', 'M. error ANN', 'M. error Digernes', 'M. error Hollenbach', 'M. error Holtrop', 'M. error Oortmerssen');
set(legend1, 'FontSize', 14);
hold off

figure(303)
line([min(Geometry.Cp), max(Geometry.Cp)], [0, 0],'LineWidth',1)
hold on
plot(Geometry.Cp, ANN.Mean_error, 'mp', Geometry.Cp, Digernes.Mean_error, 'gs', Geometry.Cp, Hollenbach.Mean_error, 'r^', Geometry.Cp, Holtrop.Mean_error, 'bo', Geometry.Cp, Oortmerssen.Mean_error, 'kd')
xlabel('C_P', 'FontSize', 14)
ylabel('Avvik [%]')
legend1 = legend('0% Error', 'M. error ANN', 'M. error Digernes', 'M. error Hollenbach', 'M. error Holtrop', 'M. error Oortmerssen');
set(legend1, 'FontSize', 14);
hold off

figure(304)
line([min(B./T), max(B./T)], [0, 0],'LineWidth',1)
hold on
plot(B./T, ANN.Mean_error, 'mp', B./T, Digernes.Mean_error, 'gs', B./T, Hollenbach.Mean_error, 'r^', B./T, Holtrop.Mean_error, 'bo', B./T, Oortmerssen.Mean_error, 'kd')
xlabel('^B/_T', 'FontSize', 14)
ylabel('Avvik [%]')
legend1 = legend('0% Error', 'M. error ANN', 'M. error Digernes', 'M. error Hollenbach', 'M. error Holtrop', 'M. error Oortmerssen');
set(legend1, 'FontSize', 14);
hold off

figure(305)
line([min(Lwl), max(Lwl)], [0, 0],'LineWidth',1)
hold on
plot(Lwl, ANN.Mean_error, 'mp', Lwl, Digernes.Mean_error, 'gs', Lwl, Hollenbach.Mean_error, 'r^', Lwl, Holtrop.Mean_error, 'bo', Lwl, Oortmerssen.Mean_error, 'kd')
xlabel('L_W_L [m]', 'FontSize', 14)
ylabel('Avvik [%]')
legend1 = legend('0% Error', 'M. error ANN', 'M. error Digernes', 'M. error Hollenbach', 'M. error Holtrop', 'M. error Oortmerssen');
set(legend1, 'FontSize', 14);
hold off

figure(306)
line([min(Tap-Tfp), max(Tap-Tfp)], [0, 0],'LineWidth',1)
hold on
plot(Tap-Tfp, ANN.Mean_error, 'mp', Tap-Tfp, Digernes.Mean_error, 'gs', Tap-Tfp, Hollenbach.Mean_error, 'r^', Tap-Tfp, Holtrop.Mean_error, 'bo', Tap-Tfp, Oortmerssen.Mean_error, 'kd')
xlabel('Trim [m]', 'FontSize', 14)
ylabel('Avvik [%]')
legend1 = legend('0% Error', 'M. error ANN', 'M. error Digernes', 'M. error Hollenbach', 'M. error Holtrop', 'M. error Oortmerssen');
set(legend1, 'FontSize', 14);
hold off

figure(307)
line([min(Geometry.Cb), max(Geometry.Cb)], [0, 0],'LineWidth',1)
hold on
plot(Geometry.Cb, ANN.Mean_error, 'mp', Geometry.Cb, Digernes.Mean_error, 'gs', Geometry.Cb, Hollenbach.Mean_error, 'r^', Geometry.Cb, Holtrop.Mean_error, 'bo', Geometry.Cb, Oortmerssen.Mean_error, 'kd')
xlabel('C_B', 'FontSize', 14)
ylabel('Avvik [%]')
legend1 = legend('0% Error', 'M. error ANN', 'M. error Digernes', 'M. error Hollenbach', 'M. error Holtrop', 'M. error Oortmerssen');
set(legend1, 'FontSize', 14);
hold off

figure(308)
line([1, n1], [0, 0],'LineWidth',1)
hold on
plot(1:n1, ANN.Mean_error, 'mp', 1:n1, Digernes.Mean_error, 'gs', 1:n1, Hollenbach.Mean_error, 'r^', 1:n1, Holtrop.Mean_error, 'bo', 1:n1, Oortmerssen.Mean_error, 'kd')
xlabel('Lpenummer - Kronologisk', 'FontSize', 14)
ylabel('Avvik [%]')
legend1 = legend('0% Error', 'M. error ANN', 'M. error Digernes', 'M. error Hollenbach', 'M. error Holtrop', 'M. error Oortmerssen');
set(legend1, 'FontSize', 14);
hold off
%%{

% Plot error trend against Froude number
figure(221)
line([min(min(Fn)), max(max(Fn))], [0, 0],'LineWidth',1.5,'Color','k')
hold on
plot(Fn, ANN.All_error, 'mp')
xlabel('F_N', 'FontSize', 14)
ylabel('Avvik [%]', 'FontSize', 14)
ylim([-100, 250])
hold off 

figure(222)
line([min(min(Fn)), max(max(Fn))], [0, 0],'LineWidth',1.5,'Color','k')
hold on
plot(Fn, Digernes.All_error, 'gs')
xlabel('F_N', 'FontSize', 14)
ylabel('Avvik [%]', 'FontSize', 14)
ylim([-100, 250])
hold off

figure(223)
line([min(min(Fn)), max(max(Fn))], [0, 0],'LineWidth',1.5,'Color','k')
hold on
plot(Fn, Hollenbach.All_error, 'r^')
xlabel('F_N', 'FontSize', 14)
ylabel('Avvik [%]', 'FontSize', 14)
ylim([-100, 250])
hold off

figure(224)
line([min(min(Fn)), max(max(Fn))], [0, 0],'LineWidth',1.5,'Color','k')
hold on
plot(Fn, Holtrop.All_error, 'bo')
xlabel('F_N', 'FontSize', 14)
ylabel('Avvik [%]', 'FontSize', 14)
ylim([-100, 250])
hold off

figure(225)
line([min(min(Fn)), max(max(Fn))], [0, 0],'LineWidth',1.5,'Color','k')
hold on
plot(Fn, Oortmerssen.All_error, 'kd')
xlabel('F_N', 'FontSize', 14)
ylabel('Avvik [%]', 'FontSize', 14)
ylim([-100, 250])
hold off

%}

clear legend1 i j Design_speed ans

%% --------------- Test Plots ------------------