function [w1, w2, w3, w4_1, w4_2, t1] = wake_est(B, L, T, Ta, Tf, S, DispV, Dp, No_P, Cb, Cp, Cm, lcb, V, k0, shape)

% Gives propulsion estimate for one vessel from different methods.

% Inputs:

% B     - Moulded breath (Bredde)
% L     - Length on waterline
% T     - Draught
% Ta    - Draugth AP
% S     - Wetted surface og hull
% DispV - Volume displacement
% Dp    - Propeller diameter
% No_P  - Number of propellers
% Cb    - Block coefficient
% Cp    - Prismatic coefficient
% Cm    - Midship section coefficient
% V     - Ship speed
% k0    - Form factor
% Shape - Shape of ship frames (string; 'U' or 'V')

% Output
% w1    - wake estimat Taylor
% w2    - wake estimate Harvald
% w3    - wake estimate Holtrop
% w4    - wake estimate British Ship Research Assosiation [1x2 vector]

% t1    - thrustdeduction estimate Holtrop


%% Taylor formula 
w1  = 0.5.*Cb - 0.05;

%% Harvald graphical method
    % Function found with "Curve fit"-app in Matlab
w2_1 = 5.597 - 35.8 *Cb - 13.51 *(B/L) + 82.82 *Cb^2 + 94.43 *Cb *(B/L) - 27.82 *(B/L)^2 - 81.32 *Cb^3 - 189.6 *Cb^2 *(B/L) + 78.46 *Cb *(B/L)^2 + 28.42 *Cb^4 + 123.1 *Cb^3 *(B/L) - 56.76 *Cb^2 *(B/L)^2;
w2_2 = 0.538 - 26.15 *(Dp/L) + 439.8 *(Dp/L)^2 - 3438 *(Dp/L)^3 + 1.004e04 *(Dp/L)^4;
if shape == 'U'
    w2_3 = -0.066 + 0.1624 *Cb - 0.009524 *Cb^2;
elseif shape == 'V'
    w2_3 = 0.05864 - 0.1269 *Cb - 0.02381 *Cb^2;
else
    w2_3 = 0;
end

%[w2_1 w2_2 w2_3]
w2      = w2_1 + w2_2 + w2_3;

%% Holtrop [1984]

c4  = Tf /L;

if (B/Ta) < 5
    c8 = (B*S)/(L*Dp*Ta);
else
    c8 = S*(7*B/Ta - 25) / (L*Dp*(B/Ta - 3));
end

if c8 < 28
    c9 = c8;
else
    c9 = 32 - 16/(c8 - 24);
end

if Ta/Dp < 2
    c11 = Ta/Dp;
else
    c11 = 0.0833333*(Ta/Dp)^3 + 1.33333;
end

if Cp < 0.7
    c19 = 0.12997 / (0.95-Cb) - 0.11056/(0.95 - Cp);
else
    c19 = 0.18567 / (1.3571 - Cm) - 0.71276 + 0.38648*Cp;
end

Cstern = -25;  % Estimated since not given in model data

c20 = 1 + 0.015*Cstern;

Cp1 = 1.45*Cp - 0.315 - 0.0225*lcb;

Rn  = (V*L)/(1.15E6);
Cf  = 0.075/((log10(Rn)-2)^2);
Ca  = 0.006 *(L+100)^(-0.16) - 0.00205 + 0.003 *sqrt(L/7.5) *Cb^4 *(0.04 - c4);

Cv  = (1+k0)*Cf + Ca;

if No_P == 1
% Single Screw ships
w3 = c9*c20*Cv*(L/Ta) *(0.050776 + 0.93405*c11 *(Cv/(1-Cp1))) + 0.27915*c20*sqrt(B/(L*(1-Cp1))) + c19*c20;

t1 = 0.25014*(B/L)^(9.28956) * (sqrt(B*T)/Dp)^(0.2624) / (1 - Cp + 0.0225*lcb)^(0.01762) + 0.0015 * Cstern;

%nR = 0.9922 - 0.05908 * (Ae/A0) + 0.07424 * (Cp - 0.0225*lcb);

elseif No_P == 2
%Twin screw ships
w3 = 0.3095*Cb + 10*Cv*Cb - 0.23 * Dp/sqrt(B*T);

t1 = 0.325*Cb - 0.1885* Dp/sqrt(B*T);

%nR = 0.9737 + 0.111*(Cp - 0.0225* lcb) - 0.06325 * P/Dp;

else
    disp('Invalid number of propellers')
end

%% British Ship Research Assosiation 
w4_1 = 0;
w4_2 = 0;

% Vk speed in knots
% Lf length in feet
% lcb in % of Lwl

Vk  = V /0.51444444;
Lf  = L*3.281;            

if No_P == 1
    Dw  = B ./sqrt(Dp .*DispV^(1/3) );
    
    if Cb >= 0.65 && Cb <= 0.8
        w4_1   = -.478 + 0.79 *Cb - 0.1 + 0.17 *Dw - 0.0072 *lcb;
    end
    if Cb >= 0.55 && Cb <= 0.8
        w4_2   = -.508 + 0.79 *Cb - 0.05 *(Vk/sqrt(Lf)) + 0.234 *Dw - 0.0028 *lcb;
    end

elseif No_P == 2
    w4_1  = 0.3095 *Cb + 10 *Cb *( (1+k0) *Cb + Cb ) -0.23 *( Dp /sqrt(B *T) );
else
    Disp('Invalid number of propellers')
end